@@ -1,17 +1,14 @@
-# -*- coding: utf-8 -*-
-import time
-import weakref
 import warnings
+import weakref
+from time import perf_counter, perf_counter_ns
 
-from ..Qt import QtCore, QtGui, QT_LIB, isQObjectAlive
-from ..Point import Point
-from .. import functions as fn
-from .. import ptime as ptime
-from .mouseEvents import *
 from .. import debug as debug
 from .. import getConfigOption
+from ..Point import Point
+from ..Qt import QT_LIB, QtCore, QtGui, QtWidgets, isQObjectAlive
+from .mouseEvents import HoverEvent, MouseClickEvent, MouseDragEvent
 
-getMillis = lambda: int(round(time.time() * 1000))
+getMillis = lambda: perf_counter_ns() // 10 ** 6
 
 
 if QT_LIB.startswith('PyQt'):
@@ -23,22 +20,22 @@ else:
 
 __all__ = ['GraphicsScene']
 
-class GraphicsScene(QtGui.QGraphicsScene):
+class GraphicsScene(QtWidgets.QGraphicsScene):
     """
     Extension of QGraphicsScene that implements a complete, parallel mouse event system.
     (It would have been preferred to just alter the way QGraphicsScene creates and delivers 
     events, but this turned out to be impossible because the constructor for QGraphicsMouseEvent
     is private)
     
-    *  Generates MouseClicked events in addition to the usual press/move/release events. 
-       (This works around a problem where it is impossible to have one item respond to a 
-       drag if another is watching for a click.)
-    *  Adjustable radius around click that will catch objects so you don't have to click *exactly* over small/thin objects
-    *  Global context menu--if an item implements a context menu, then its parent(s) may also add items to the menu.
-    *  Allows items to decide _before_ a mouse click which item will be the recipient of mouse events.
-       This lets us indicate unambiguously to the user which item they are about to click/drag on
-    *  Eats mouseMove events that occur too soon after a mouse press.
-    *  Reimplements items() and itemAt() to circumvent PyQt bug
+      *  Generates MouseClicked events in addition to the usual press/move/release events.
+         (This works around a problem where it is impossible to have one item respond to a
+         drag if another is watching for a click.)
+      *  Adjustable radius around click that will catch objects so you don't have to click *exactly* over small/thin objects
+      *  Global context menu--if an item implements a context menu, then its parent(s) may also add items to the menu.
+      *  Allows items to decide _before_ a mouse click which item will be the recipient of mouse events.
+         This lets us indicate unambiguously to the user which item they are about to click/drag on
+      *  Eats mouseMove events that occur too soon after a mouse press.
+      *  Reimplements items() and itemAt() to circumvent PyQt bug
 
     ====================== ====================================================================
     **Signals**
@@ -102,7 +99,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
         )
 
     def __init__(self, clickRadius=2, moveDistance=5, parent=None):
-        QtGui.QGraphicsScene.__init__(self, parent)
+        QtWidgets.QGraphicsScene.__init__(self, parent)
         self.setClickRadius(clickRadius)
         self.setMoveDistance(moveDistance)
         self.exportDirectory = None
@@ -124,7 +121,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
         
     def render(self, *args):
         self.prepareForPaint()
-        return QtGui.QGraphicsScene.render(self, *args)
+        return QtWidgets.QGraphicsScene.render(self, *args)
 
     def prepareForPaint(self):
         """Called before every render. This method will inform items that the scene is about to
@@ -198,7 +195,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
                 # button is pressed' send mouseMoveEvents and mouseDragEvents
                 super().mouseMoveEvent(ev)
                 if self.mouseGrabberItem() is None:
-                    now = ptime.time()
+                    now = perf_counter()
                     init = False
                     ## keep track of which buttons are involved in dragging
                     for btn in [QtCore.Qt.MouseButton.LeftButton, QtCore.Qt.MouseButton.MiddleButton, QtCore.Qt.MouseButton.RightButton]:
@@ -397,26 +394,26 @@ class GraphicsScene(QtGui.QGraphicsScene):
 
     def addItem(self, item):
         # extend QGraphicsScene.addItem to emit a sigItemAdded signal
-        ret = QtGui.QGraphicsScene.addItem(self, item)
+        ret = QtWidgets.QGraphicsScene.addItem(self, item)
         self.sigItemAdded.emit(item)
         return ret
 
     def removeItem(self, item):
         # extend QGraphicsScene.removeItem to emit a sigItemRemoved signal
-        ret = QtGui.QGraphicsScene.removeItem(self, item)
+        ret = QtWidgets.QGraphicsScene.removeItem(self, item)
         self.sigItemRemoved.emit(item)
         return ret
         
     def items(self, *args):
-        items = QtGui.QGraphicsScene.items(self, *args)
+        items = QtWidgets.QGraphicsScene.items(self, *args)
         return self.translateGraphicsItems(items)
     
     def selectedItems(self, *args):
-        items = QtGui.QGraphicsScene.selectedItems(self, *args)
+        items = QtWidgets.QGraphicsScene.selectedItems(self, *args)
         return self.translateGraphicsItems(items)
 
     def itemAt(self, *args):
-        item = QtGui.QGraphicsScene.itemAt(self, *args)
+        item = QtWidgets.QGraphicsScene.itemAt(self, *args)
         return self.translateGraphicsItem(item)
 
     def itemsNearEvent(self, event, selMode=QtCore.Qt.ItemSelectionMode.IntersectsItemShape, sortOrder=QtCore.Qt.SortOrder.DescendingOrder, hoverable=False):
@@ -485,10 +482,10 @@ class GraphicsScene(QtGui.QGraphicsScene):
         ### same pyqt bug -- mouseEvent.widget() doesn't give us the original python object.
         ### [[doesn't seem to work correctly]]
         #if HAVE_SIP and isinstance(self, sip.wrapper):
-            #addr = sip.unwrapinstance(sip.cast(widget, QtGui.QWidget))
+            #addr = sip.unwrapinstance(sip.cast(widget, QtWidgets.QWidget))
             ##print "convert", widget, addr
             #for v in self.views():
-                #addr2 = sip.unwrapinstance(sip.cast(v, QtGui.QWidget))
+                #addr2 = sip.unwrapinstance(sip.cast(v, QtWidgets.QWidget))
                 ##print "   check:", v, addr2
                 #if addr2 == addr:
                     #return v
@@ -542,7 +539,7 @@ class GraphicsScene(QtGui.QGraphicsScene):
             menu.addSeparator()
 
         for m in menusToAdd:
-            if isinstance(m, QtGui.QMenu):
+            if isinstance(m, QtWidgets.QMenu):
                 menu.addAction(m.menuAction())
             elif isinstance(m, QtGui.QAction):
                 menu.addAction(m)
