@@ -1,19 +1,50 @@
-# -*- coding: utf-8 -*-
 """
 debug.py - Functions to aid in debugging 
 Copyright 2010  Luke Campagnola
 Distributed under MIT/X11 license. See license.txt for more information.
 """
 
+
 from __future__ import print_function
 
-import sys, traceback, time, gc, re, types, weakref, inspect, os, cProfile, threading
-from . import ptime
+import contextlib
+import cProfile
+import gc
+import inspect
+import os
+import re
+import sys
+import threading
+import time
+import traceback
+import types
+import warnings
+import weakref
+from time import perf_counter
+
 from numpy import ndarray
-from .Qt import QtCore, QtGui
-from .util.mutex import Mutex
+
+from .Qt import QT_LIB, QtCore
 from .util import cprint
 
+if sys.version.startswith("3.8") and QT_LIB == "PySide2":
+    from .Qt import PySide2
+    if tuple(map(int, PySide2.__version__.split("."))) < (5, 14):
+        warnings.warn("Due to PYSIDE-1140, ThreadChase and ThreadColor won't work")
+from .util.mutex import Mutex
+
+
+# credit to Wolph: https://stackoverflow.com/a/17603000
+@contextlib.contextmanager
+def open_maybe_console(filename=None):
+    fh = sys.stdout if filename is None else open(filename, "w", encoding='utf-8')
+    try:
+        yield fh
+    finally:
+        if fh is not sys.stdout:
+            fh.close()
+
+
 __ftraceDepth = 0
 def ftrace(func):
     """Decorator used for marking the beginning and end of function calls.
@@ -106,11 +137,10 @@ def getExc(indent=4, prefix='|  ', skip=1):
 def printExc(msg='', indent=4, prefix='|'):
     """Print an error message followed by an indented exception backtrace
     (This function is intended to be called within except: blocks)"""
-    exc = getExc(indent, prefix + '  ', skip=2)
-    print("[%s]  %s\n" % (time.strftime("%H:%M:%S"), msg))
-    print(" "*indent + prefix + '='*30 + '>>')
-    print(exc)
-    print(" "*indent + prefix + '='*30 + '<<')
+    exc = getExc(indent=0, prefix="", skip=2)
+    # print(" "*indent + prefix + '='*30 + '>>')
+    warnings.warn("\n".join([msg, exc]), RuntimeWarning, stacklevel=2)
+    # print(" "*indent + prefix + '='*30 + '<<')
 
     
 def printTrace(msg='', indent=4, prefix='|'):
@@ -161,13 +191,15 @@ def listObjs(regex='Q', typ=None):
         
 
     
-def findRefPath(startObj, endObj, maxLen=8, restart=True, seen={}, path=None, ignore=None):
+def findRefPath(startObj, endObj, maxLen=8, restart=True, seen=None, path=None, ignore=None):
     """Determine all paths of object references from startObj to endObj"""
     refs = []
     if path is None:
         path = [endObj]
     if ignore is None:
         ignore = {}
+    if seen is None:
+        seen = {}
     ignore[id(sys._getframe())] = None
     ignore[id(path)] = None
     ignore[id(seen)] = None
@@ -195,7 +227,7 @@ def findRefPath(startObj, endObj, maxLen=8, restart=True, seen={}, path=None, ig
             #print prefix+"  FRAME"
             continue
         try:
-            if any([r is x for x in  path]):
+            if any(r is x for x in  path):
                 #print prefix+"  LOOP", objChainString([r]+path)
                 continue
         except:
@@ -275,7 +307,7 @@ def refPathString(chain):
         o2 = chain[i]
         cont = False
         if isinstance(o1, list) or isinstance(o1, tuple):
-            if any([o2 is x for x in o1]):
+            if any(o2 is x for x in o1):
                 s += "[%d]" % o1.index(o2)
                 continue
         #print "  not list"
@@ -523,7 +555,7 @@ class Profiler(object):
         obj._delayed = delayed
         obj._markCount = 0
         obj._finished = False
-        obj._firstTime = obj._lastTime = ptime.time()
+        obj._firstTime = obj._lastTime = perf_counter()
         obj._newMsg("> Entering " + obj._name)
         return obj
 
@@ -535,7 +567,7 @@ class Profiler(object):
         if msg is None:
             msg = str(self._markCount)
         self._markCount += 1
-        newTime = ptime.time()
+        newTime = perf_counter()
         self._newMsg("  %s: %0.4f ms", 
                      msg, (newTime - self._lastTime) * 1000)
         self._lastTime = newTime
@@ -563,7 +595,7 @@ class Profiler(object):
         if msg is not None:
             self(msg)
         self._newMsg("< Exiting %s, total time: %0.4f ms", 
-                     self._name, (ptime.time() - self._firstTime) * 1000)
+                     self._name, (perf_counter() - self._firstTime) * 1000)
         type(self)._depth -= 1
         if self._depth < 1:
             self.flush()
@@ -576,6 +608,7 @@ class Profiler(object):
 
 def profile(code, name='profile_run', sort='cumulative', num=30):
     """Common-use for cProfile"""
+    import pstats
     cProfile.run(code, name)
     stats = pstats.Stats(name)
     stats.sort_stats(sort)
@@ -794,7 +827,7 @@ class ObjTracker(object):
                 continue
             
             try:
-                ref = weakref.ref(obj)
+                ref = weakref.ref(o)
             except:
                 ref = None
             refs[oid] = ref
@@ -1126,10 +1159,11 @@ class ThreadTrace(object):
     Used to debug freezing by starting a new thread that reports on the 
     location of other threads periodically.
     """
-    def __init__(self, interval=10.0):
+    def __init__(self, interval=10.0, logFile=None):
         self.interval = interval
         self.lock = Mutex()
         self._stop = False
+        self.logFile = logFile
         self.start()
 
     def stop(self):
@@ -1145,35 +1179,41 @@ class ThreadTrace(object):
         self.thread.start()
 
     def run(self):
-        while True:
-            with self.lock:
-                if self._stop is True:
-                    return
-                    
-            print("\n=============  THREAD FRAMES:  ================")
-            for id, frame in sys._current_frames().items():
-                if id == threading.current_thread().ident:
-                    continue
+        iter = 0
+        with open_maybe_console(self.logFile) as printFile:
+            while True:
+                with self.lock:
+                    if self._stop is True:
+                        return
 
-                # try to determine a thread name
-                try:
-                    name = threading._active.get(id, None)
-                except:
-                    name = None
-                if name is None:
+                printFile.write(f"\n=============  THREAD FRAMES {iter}:  ================\n")
+                for id, frame in sys._current_frames().items():
+                    if id == threading.current_thread().ident:
+                        continue
+
+                    # try to determine a thread name
                     try:
-                        # QThread._names must be manually set by thread creators.
-                        name = QtCore.QThread._names.get(id)
+                        name = threading._active.get(id, None)
                     except:
                         name = None
-                if name is None:
-                    name = "???"
+                    if name is None:
+                        try:
+                            # QThread._names must be manually set by thread creators.
+                            name = QtCore.QThread._names.get(id)
+                        except:
+                            name = None
+                    if name is None:
+                        name = "???"
 
-                print("<< thread %d \"%s\" >>" % (id, name))
-                traceback.print_stack(frame)
-            print("===============================================\n")
-            
-            time.sleep(self.interval)
+                    printFile.write("<< thread %d \"%s\" >>\n" % (id, name))
+                    tb = str(''.join(traceback.format_stack(frame)))
+                    printFile.write(tb)
+                    printFile.write("\n")
+                printFile.write("===============================================\n\n")
+                printFile.flush()
+
+                iter += 1
+                time.sleep(self.interval)
 
 
 class ThreadColor(object):
@@ -1217,10 +1257,10 @@ def enableFaulthandler():
     """
     try:
         import faulthandler
+
         # necessary to disable first or else new threads may not be handled.
         faulthandler.disable()
         faulthandler.enable(all_threads=True)
         return True
     except ImportError:
         return False
-
