@@ -1,25 +1,17 @@
-# -*- coding: utf-8 -*-
-from ..Qt import QtCore, QtGui
-from .Container import *
-from .DockDrop import *
-from .Dock import Dock
-from .. import debug as debug
 import weakref
 
-## TODO:
-# - containers should be drop areas, not docks. (but every slot within a container must have its own drop areas?)
-# - drop between tabs
-# - nest splitters inside tab boxes, etc.
+from ..Qt import QtWidgets
+from .Container import Container, HContainer, TContainer, VContainer
+from .Dock import Dock
+from .DockDrop import DockDrop
 
 
-
-
-class DockArea(Container, QtGui.QWidget, DockDrop):
-    def __init__(self, temporary=False, home=None):
+class DockArea(Container, QtWidgets.QWidget, DockDrop):
+    def __init__(self, parent=None, temporary=False, home=None):
         Container.__init__(self, self)
-        QtGui.QWidget.__init__(self)
+        QtWidgets.QWidget.__init__(self, parent=parent)
         DockDrop.__init__(self, allowedAreas=['left', 'right', 'top', 'bottom'])
-        self.layout = QtGui.QVBoxLayout()
+        self.layout = QtWidgets.QVBoxLayout()
         self.layout.setContentsMargins(0,0,0,0)
         self.layout.setSpacing(0)
         self.setLayout(self.layout)
@@ -52,6 +44,10 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         """
         if dock is None:
             dock = Dock(**kwds)
+            
+        # store original area that the dock will return to when un-floated
+        if not self.temporary:
+            dock.orig_area = self
         
         
         ## Determine the container to insert this dock into.
@@ -64,9 +60,11 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
                 container = self.topContainer
                 neighbor = None
         else:
-            if isinstance(relativeTo, basestring):
+            if isinstance(relativeTo, str):
                 relativeTo = self.docks[relativeTo]
             container = self.getContainer(relativeTo)
+            if container is None:
+                raise TypeError("Dock %s is not contained in a DockArea; cannot add another dock relative to it." % relativeTo)
             neighbor = relativeTo
         
         ## what container type do we need?
@@ -102,9 +100,11 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
             'below': 'after'
         }[position]
         #print "request insert", dock, insertPos, neighbor
+        old = dock.container()
         container.insert(dock, insertPos, neighbor)
-        dock.area = self
         self.docks[dock.name()] = dock
+        if old is not None:
+            old.apoptose()
         
         return dock
         
@@ -112,12 +112,10 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         """
         Move an existing Dock to a new location. 
         """
-        old = dock.container()
         ## Moving to the edge of a tabbed dock causes a drop outside the tab box
         if position in ['left', 'right', 'top', 'bottom'] and neighbor is not None and neighbor.container() is not None and neighbor.container().type() == 'tab':
             neighbor = neighbor.container()
         self.addDock(dock, position, neighbor)
-        old.apoptose()
         
     def getContainer(self, obj):
         if obj is None:
@@ -131,6 +129,8 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
             new = HContainer(self)
         elif typ == 'tab':
             new = TContainer(self)
+        else:
+            raise ValueError("typ must be one of 'vertical', 'horizontal', or 'tab'")
         return new
         
     def addContainer(self, typ, obj):
@@ -147,23 +147,19 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
     
     def insert(self, new, pos=None, neighbor=None):
         if self.topContainer is not None:
+            # Adding new top-level container; addContainer() should
+            # take care of giving the old top container a new home.
             self.topContainer.containerChanged(None)
         self.layout.addWidget(new)
+        new.containerChanged(self)
         self.topContainer = new
-        #print self, "set top:", new
-        new._container = self
         self.raiseOverlay()
-        #print "Insert top:", new
         
     def count(self):
         if self.topContainer is None:
             return 0
         return 1
         
-        
-    #def paintEvent(self, ev):
-        #self.drawDockOverlay()
-        
     def resizeEvent(self, ev):
         self.resizeOverlay(self.size())
         
@@ -171,8 +167,7 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         if self.home is None:
             area = DockArea(temporary=True, home=self)
             self.tempAreas.append(area)
-            win = QtGui.QMainWindow()
-            win.setCentralWidget(area)
+            win = TempAreaWindow(area)
             area.win = win
             win.show()
         else:
@@ -186,7 +181,6 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         area.win.resize(dock.size())
         area.moveDock(dock, 'top', None)
         
-        
     def removeTempArea(self, area):
         self.tempAreas.remove(area)
         #print "close window", area.window()
@@ -196,7 +190,13 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         """
         Return a serialized (storable) representation of the state of
         all Docks in this DockArea."""
-        state = {'main': self.childState(self.topContainer), 'float': []}
+
+        if self.topContainer is None:
+            main = None
+        else:
+            main = self.childState(self.topContainer)
+
+        state = {'main': main, 'float': []}
         for a in self.tempAreas:
             geo = a.win.geometry()
             geo = (geo.x(), geo.y(), geo.width(), geo.height())
@@ -212,14 +212,20 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
                 childs.append(self.childState(obj.widget(i)))
             return (obj.type(), childs, obj.saveState())
         
-        
-    def restoreState(self, state):
+    def restoreState(self, state, missing='error', extra='bottom'):
         """
         Restore Dock configuration as generated by saveState.
         
-        Note that this function does not create any Docks--it will only 
+        This function does not create any Docks--it will only 
         restore the arrangement of an existing set of Docks.
         
+        By default, docks that are described in *state* but do not exist
+        in the dock area will cause an exception to be raised. This behavior
+        can be changed by setting *missing* to 'ignore' or 'create'.
+        
+        Extra docks that are in the dockarea but that are not mentioned in
+        *state* will be added to the bottom of the dockarea, unless otherwise
+        specified by the *extra* argument.
         """
         
         ## 1) make dict of all docks and list of existing containers
@@ -228,17 +234,23 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         #print "found docks:", docks
         
         ## 2) create container structure, move docks into new containers
-        self.buildFromState(state['main'], docks, self)
+        if state['main'] is not None:
+            self.buildFromState(state['main'], docks, self, missing=missing)
         
         ## 3) create floating areas, populate
         for s in state['float']:
             a = self.addTempArea()
-            a.buildFromState(s[0]['main'], docks, a)
+            a.buildFromState(s[0]['main'], docks, a, missing=missing)
             a.win.setGeometry(*s[1])
+            a.apoptose()  # ask temp area to close itself if it is empty
         
-        ## 4) Add any remaining docks to the bottom
+        ## 4) Add any remaining docks to a float
         for d in docks.values():
-            self.moveDock(d, 'below', None)
+            if extra == 'float':
+                a = self.addTempArea()
+                a.addDock(d, 'below')
+            else:
+                self.moveDock(d, extra, None)
         
         #print "\nKill old containers:"
         ## 5) kill old containers
@@ -247,8 +259,7 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         for a in oldTemps:
             a.apoptose()
 
-
-    def buildFromState(self, state, docks, root, depth=0):
+    def buildFromState(self, state, docks, root, depth=0, missing='error'):
         typ, contents, state = state
         pfx = "  " * depth
         if typ == 'dock':
@@ -256,7 +267,15 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
                 obj = docks[contents]
                 del docks[contents]
             except KeyError:
-                raise Exception('Cannot restore dock state; no dock with name "%s"' % contents)
+                if missing == 'error':
+                    raise Exception('Cannot restore dock state; no dock with name "%s"' % contents)
+                elif missing == 'create':
+                    obj = Dock(name=contents)
+                elif missing == 'ignore':
+                    return
+                else:
+                    raise ValueError('"missing" argument must be one of "error", "create", or "ignore".')
+
         else:
             obj = self.makeContainer(typ)
             
@@ -265,10 +284,11 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
         
         if typ != 'dock':
             for o in contents:
-                self.buildFromState(o, docks, obj, depth+1)
+                self.buildFromState(o, docks, obj, depth+1, missing=missing)
+            # remove this container if possible. (there are valid situations when a restore will
+            # generate empty containers, such as when using missing='ignore')
             obj.apoptose(propagate=False)
-            obj.restoreState(state)  ## this has to be done later?
-        
+            obj.restoreState(state)  ## this has to be done later?     
 
     def findAll(self, obj=None, c=None, d=None):
         if obj is None:
@@ -294,12 +314,19 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
                 d.update(d2)
         return (c, d)
 
-    def apoptose(self):
+    def apoptose(self, propagate=True):
+        # remove top container if possible, close this area if it is temporary.
         #print "apoptose area:", self.temporary, self.topContainer, self.topContainer.count()
-        if self.temporary and self.topContainer.count() == 0:
+        if self.topContainer is None or self.topContainer.count() == 0:
             self.topContainer = None
-            self.home.removeTempArea(self)
-            #self.close()
+            if self.temporary and self.home is not None:
+                self.home.removeTempArea(self)
+                #self.close()
+                
+    def clear(self):
+        docks = self.findAll()[1]
+        for dock in docks.values():
+            dock.close()
             
     ## PySide bug: We need to explicitly redefine these methods
     ## or else drag/drop events will not be delivered.
@@ -315,5 +342,44 @@ class DockArea(Container, QtGui.QWidget, DockDrop):
     def dropEvent(self, *args):
         DockDrop.dropEvent(self, *args)
 
-        
-        
+    def printState(self, state=None, name='Main'):
+        # for debugging
+        if state is None:
+            state = self.saveState()
+        print("=== %s dock area ===" % name)
+        if state['main'] is None:
+            print("   (empty)")
+        else:
+            self._printAreaState(state['main'])
+        for i, float in enumerate(state['float']):
+            self.printState(float[0], name='float %d' % i)
+
+    def _printAreaState(self, area, indent=0):
+        if area[0] == 'dock':
+            print("  " * indent + area[0] + " " + str(area[1:]))
+            return
+        else:
+            print("  " * indent + area[0])
+            for ch in area[1]:
+                self._printAreaState(ch, indent+1)
+
+
+
+class TempAreaWindow(QtWidgets.QWidget):
+    def __init__(self, area, **kwargs):
+        QtWidgets.QWidget.__init__(self, **kwargs)
+        self.layout = QtWidgets.QGridLayout()
+        self.setLayout(self.layout)
+        self.layout.setContentsMargins(0, 0, 0, 0)
+        self.dockarea = area
+        self.layout.addWidget(area)
+
+    def closeEvent(self, *args):
+        # restore docks to their original area
+        docks = self.dockarea.findAll()[1]
+        for dock in docks.values():
+            if hasattr(dock, 'orig_area'):
+                dock.orig_area.addDock(dock, )
+        # clear dock area, and close remaining docks
+        self.dockarea.clear()
+        super().closeEvent(*args)
