@@ -1,23 +1,26 @@
-# -*- coding: utf-8 -*-
-from ..Qt import QtCore, QtGui, QT_LIB
-from .Node import *
-from collections import OrderedDict
-from ..widgets.TreeWidget import *
-from .. import FileDialog, DataTreeWidget
+__init__ = ["Flowchart", "FlowchartGraphicsItem", "FlowchartNode"]
 
 import importlib
+from collections import OrderedDict
+
+from .. import DataTreeWidget, FileDialog
+from ..Qt import QT_LIB, QtCore, QtWidgets
+from .Node import Node
+
 FlowchartCtrlTemplate = importlib.import_module(
     f'.FlowchartCtrlTemplate_{QT_LIB.lower()}', package=__package__)
     
-from .Terminal import Terminal
 from numpy import ndarray
-from .library import LIBRARY
-from ..debug import printExc
+
 from .. import configfile as configfile
 from .. import dockarea as dockarea
-from . import FlowchartGraphicsView
 from .. import functions as fn
-from ..python2_3 import asUnicode
+from ..debug import printExc
+from ..graphicsItems.GraphicsObject import GraphicsObject
+from . import FlowchartGraphicsView
+from .library import LIBRARY
+from .Terminal import Terminal
+
 
 def strDict(d):
     return dict([(str(k), v) for k, v in d.items()])
@@ -431,7 +434,7 @@ class Flowchart(Node):
         conn = set()
         for n in self._nodes.values():
             terms = n.outputs()
-            for n, t in terms.items():
+            for t in terms.values():
                 for c in t.connections():
                     conn.add((t, c))
         return conn
@@ -512,7 +515,6 @@ class Flowchart(Node):
             self.fileDialog.fileSelected.connect(self.loadFile)
             return
             ## NOTE: was previously using a real widget for the file dialog's parent, but this caused weird mouse event bugs..
-        fileName = asUnicode(fileName)
         state = configfile.readConfigFile(fileName)
         self.restoreState(state, clear=True)
         self.viewBox.autoRange()
@@ -528,11 +530,10 @@ class Flowchart(Node):
                 startDir = '.'
             self.fileDialog = FileDialog(None, "Save Flowchart..", startDir, "Flowchart (*.fc)")
             self.fileDialog.setDefaultSuffix("fc")
-            self.fileDialog.setAcceptMode(QtGui.QFileDialog.AcceptMode.AcceptSave) 
+            self.fileDialog.setAcceptMode(QtWidgets.QFileDialog.AcceptMode.AcceptSave) 
             self.fileDialog.show()
             self.fileDialog.fileSelected.connect(self.saveFile)
             return
-        fileName = asUnicode(fileName)
         configfile.writeConfigFile(self.saveState(), fileName)
         self.sigFileSaved.emit(fileName)
 
@@ -591,14 +592,14 @@ class FlowchartGraphicsItem(GraphicsObject):
         #p.drawRect(self.boundingRect())
     
 
-class FlowchartCtrlWidget(QtGui.QWidget):
+class FlowchartCtrlWidget(QtWidgets.QWidget):
     """The widget that contains the list of all the nodes in a flowchart and their controls, as well as buttons for loading/saving flowcharts."""
     
     def __init__(self, chart):
         self.items = {}
         #self.loadDir = loadDir  ## where to look initially for chart files
         self.currentFileName = None
-        QtGui.QWidget.__init__(self)
+        QtWidgets.QWidget.__init__(self)
         self.chart = chart
         self.ui = FlowchartCtrlTemplate.Ui_Form()
         self.ui.setupUi(self)
@@ -610,7 +611,7 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         
         self.chartWidget = FlowchartWidget(chart, self)
         #self.chartWidget.viewBox().autoRange()
-        self.cwWin = QtGui.QMainWindow()
+        self.cwWin = QtWidgets.QMainWindow()
         self.cwWin.setWindowTitle('Flowchart')
         self.cwWin.setCentralWidget(self.chartWidget)
         self.cwWin.resize(1000,800)
@@ -630,7 +631,7 @@ class FlowchartCtrlWidget(QtGui.QWidget):
     
         
     #def resizeEvent(self, ev):
-        #QtGui.QWidget.resizeEvent(self, ev)
+        #QtWidgets.QWidget.resizeEvent(self, ev)
         #self.ui.ctrlList.setColumnWidth(0, self.ui.ctrlList.viewport().width()-20)
         
     def chartToggled(self, b):
@@ -653,7 +654,7 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         #self.setCurrentFile(newFile)
         
     def fileSaved(self, fileName):
-        self.setCurrentFile(asUnicode(fileName))
+        self.setCurrentFile(fileName)
         self.ui.saveBtn.success("Saved.")
         
     def saveClicked(self):
@@ -682,7 +683,7 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         #self.setCurrentFile(newFile)
             
     def setCurrentFile(self, fileName):
-        self.currentFileName = asUnicode(fileName)
+        self.currentFileName = fileName
         if fileName is None:
             self.ui.fileNameLabel.setText("<b>[ new ]</b>")
         else:
@@ -705,9 +706,9 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         ctrl = node.ctrlWidget()
         #if ctrl is None:
             #return
-        item = QtGui.QTreeWidgetItem([node.name(), '', ''])
+        item = QtWidgets.QTreeWidgetItem([node.name(), '', ''])
         self.ui.ctrlList.addTopLevelItem(item)
-        byp = QtGui.QPushButton('X')
+        byp = QtWidgets.QPushButton('X')
         byp.setCheckable(True)
         byp.setFixedWidth(20)
         item.bypassBtn = byp
@@ -718,7 +719,7 @@ class FlowchartCtrlWidget(QtGui.QWidget):
         byp.clicked.connect(self.bypassClicked)
         
         if ctrl is not None:
-            item2 = QtGui.QTreeWidgetItem()
+            item2 = QtWidgets.QTreeWidgetItem()
             item.addChild(item2)
             self.ui.ctrlList.setItemWidget(item2, 0, ctrl)
             
@@ -759,13 +760,13 @@ class FlowchartCtrlWidget(QtGui.QWidget):
 class FlowchartWidget(dockarea.DockArea):
     """Includes the actual graphical flowchart and debugging interface"""
     def __init__(self, chart, ctrl):
-        #QtGui.QWidget.__init__(self)
+        #QtWidgets.QWidget.__init__(self)
         dockarea.DockArea.__init__(self)
         self.chart = chart
         self.ctrl = ctrl
         self.hoverItem = None
         #self.setMinimumWidth(250)
-        #self.setSizePolicy(QtGui.QSizePolicy(QtGui.QSizePolicy.Policy.Preferred, QtGui.QSizePolicy.Policy.Expanding))
+        #self.setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Expanding))
         
         #self.ui = FlowchartTemplate.Ui_Form()
         #self.ui.setupUi(self)
@@ -778,17 +779,17 @@ class FlowchartWidget(dockarea.DockArea):
         self.addDock(self.viewDock)
     
 
-        self.hoverText = QtGui.QTextEdit()
+        self.hoverText = QtWidgets.QTextEdit()
         self.hoverText.setReadOnly(True)
         self.hoverDock = dockarea.Dock('Hover Info', size=(1000,20))
         self.hoverDock.addWidget(self.hoverText)
         self.addDock(self.hoverDock, 'bottom')
 
-        self.selInfo = QtGui.QWidget()
-        self.selInfoLayout = QtGui.QGridLayout()
+        self.selInfo = QtWidgets.QWidget()
+        self.selInfoLayout = QtWidgets.QGridLayout()
         self.selInfo.setLayout(self.selInfoLayout)
-        self.selDescLabel = QtGui.QLabel()
-        self.selNameLabel = QtGui.QLabel()
+        self.selDescLabel = QtWidgets.QLabel()
+        self.selNameLabel = QtWidgets.QLabel()
         self.selDescLabel.setWordWrap(True)
         self.selectedTree = DataTreeWidget()
         #self.selectedTree.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarPolicy.ScrollBarAsNeeded)
@@ -801,7 +802,7 @@ class FlowchartWidget(dockarea.DockArea):
         
         self._scene = self.view.scene()
         self._viewBox = self.view.viewBox()
-        #self._scene = QtGui.QGraphicsScene()
+        #self._scene = QtWidgets.QGraphicsScene()
         #self._scene = FlowchartGraphicsView.FlowchartGraphicsScene()
         #self.view.setScene(self._scene)
         
@@ -827,7 +828,7 @@ class FlowchartWidget(dockarea.DockArea):
         def buildSubMenu(node, rootMenu, subMenus, pos=None):
             for section, node in node.items():
                 if isinstance(node, OrderedDict):
-                    menu = QtGui.QMenu(section)
+                    menu = QtWidgets.QMenu(section)
                     rootMenu.addMenu(menu)
                     buildSubMenu(node, menu, subMenus, pos=pos)
                     subMenus.append(menu)
@@ -835,7 +836,7 @@ class FlowchartWidget(dockarea.DockArea):
                     act = rootMenu.addAction(section)
                     act.nodeType = section
                     act.pos = pos
-        self.nodeMenu = QtGui.QMenu()
+        self.nodeMenu = QtWidgets.QMenu()
         self.subMenus = []       
         buildSubMenu(self.chart.library.getNodeTree(), self.nodeMenu, self.subMenus, pos=pos)
         self.nodeMenu.triggered.connect(self.nodeMenuTriggered)
@@ -845,7 +846,7 @@ class FlowchartWidget(dockarea.DockArea):
         self.menuPos = pos
     
     def showViewMenu(self, ev):
-        #QtGui.QPushButton.mouseReleaseEvent(self.ui.addNodeBtn, ev)
+        #QtWidgets.QPushButton.mouseReleaseEvent(self.ui.addNodeBtn, ev)
         #if ev.button() == QtCore.Qt.MouseButton.RightButton:
             #self.menuPos = self.view.mapToScene(ev.pos())
             #self.nodeMenu.popup(ev.globalPos())
