@@ -1,12 +1,14 @@
-# -*- coding: utf-8 -*-
-from ..Qt import QtCore, QtGui, QtWidgets
-from ..graphicsItems.GraphicsObject import GraphicsObject
-from .. import functions as fn
-from .Terminal import *
-from collections import OrderedDict
-from ..debug import *
-import numpy as np
+__all__ = ["Node", "NodeGraphicsItem"]
+
+import sys
 import warnings
+from collections import OrderedDict
+
+from .. import functions as fn
+from ..debug import printExc
+from ..graphicsItems.GraphicsObject import GraphicsObject
+from ..Qt import QtCore, QtGui, QtWidgets
+from .Terminal import Terminal
 
 translate = QtCore.QCoreApplication.translate
 
@@ -18,7 +20,7 @@ class Node(QtCore.QObject):
     Node represents the basic processing unit of a flowchart. 
     A Node subclass implements at least:
     
-    1) A list of input / ouptut terminals and their properties
+    1) A list of input / output terminals and their properties
     2) a process() function which takes the names of input terminals as keyword arguments and returns a dict with the names of output terminals as keys.
 
     A flowchart thus consists of multiple instances of Node subclasses, each of which is connected
@@ -462,14 +464,14 @@ class TextItem(QtWidgets.QGraphicsTextItem):
         super().keyPressEvent(ev)
 
 
-#class NodeGraphicsItem(QtGui.QGraphicsItem):
+#class NodeGraphicsItem(QtWidgets.QGraphicsItem):
 class NodeGraphicsItem(GraphicsObject):
     def __init__(self, node):
-        #QtGui.QGraphicsItem.__init__(self)
+        #QtWidgets.QGraphicsItem.__init__(self)
         GraphicsObject.__init__(self)
         #QObjectWorkaround.__init__(self)
         
-        #self.shadow = QtGui.QGraphicsDropShadowEffect()
+        #self.shadow = QtWidgets.QGraphicsDropShadowEffect()
         #self.shadow.setOffset(5,5)
         #self.shadow.setBlurRadius(10)
         #self.setGraphicsEffect(self.shadow)
@@ -505,7 +507,7 @@ class NodeGraphicsItem(GraphicsObject):
         #GraphicsObject.setZValue(self, z)
         
     def labelChanged(self):
-        newName = str(self.nameItem.toPlainText())
+        newName = self.nameItem.toPlainText()
         if newName != self.node.name():
             self.node.rename(newName)
             
@@ -591,7 +593,7 @@ class NodeGraphicsItem(GraphicsObject):
             ev.accept()
             #print "    ev.button: left"
             sel = self.isSelected()
-            #ret = QtGui.QGraphicsItem.mousePressEvent(self, ev)
+            #ret = QtWidgets.QGraphicsItem.mousePressEvent(self, ev)
             self.setSelected(True)
             if not sel and self.isSelected():
                 #self.setBrush(QtGui.QBrush(QtGui.QColor(200, 200, 255)))
@@ -646,7 +648,7 @@ class NodeGraphicsItem(GraphicsObject):
         menu.popup(QtCore.QPoint(pos.x(), pos.y()))
         
     def buildMenu(self):
-        self.menu = QtGui.QMenu()
+        self.menu = QtWidgets.QMenu()
         self.menu.setTitle(translate("Context Menu", "Node"))
         a = self.menu.addAction(translate("Context Menu","Add input"), self.addInputFromMenu)
         if not self.node._allowAddInput:
