@@ -1,12 +1,14 @@
-from ..Qt import QtGui, QtCore
+from math import atan2, degrees
+
+import numpy as np
+
+from .. import functions as fn
 from ..Point import Point
+from ..Qt import QtCore, QtGui
+from .GraphicsItem import GraphicsItem
 from .GraphicsObject import GraphicsObject
 from .TextItem import TextItem
 from .ViewBox import ViewBox
-from .. import functions as fn
-import numpy as np
-import weakref
-
 
 __all__ = ['InfiniteLine', 'InfLineLabel']
 
@@ -23,15 +25,18 @@ class InfiniteLine(GraphicsObject):
     sigDragged(self)
     sigPositionChangeFinished(self)
     sigPositionChanged(self)
+    sigClicked(self, ev)
     =============================== ===================================================
     """
 
     sigDragged = QtCore.Signal(object)
     sigPositionChangeFinished = QtCore.Signal(object)
     sigPositionChanged = QtCore.Signal(object)
+    sigClicked = QtCore.Signal(object, object)
 
     def __init__(self, pos=None, angle=90, pen=None, movable=False, bounds=None,
-                 hoverPen=None, label=None, labelOpts=None, name=None):
+                 hoverPen=None, label=None, labelOpts=None, span=(0, 1), markers=None, 
+                 name=None):
         """
         =============== ==================================================================
         **Arguments:**
@@ -41,22 +46,28 @@ class InfiniteLine(GraphicsObject):
         pen             Pen to use when drawing line. Can be any arguments that are valid
                         for :func:`mkPen <pyqtgraph.mkPen>`. Default pen is transparent
                         yellow.
+        hoverPen        Pen to use when the mouse cursor hovers over the line. 
+                        Only used when movable=True.
         movable         If True, the line can be dragged to a new position by the user.
+        bounds          Optional [min, max] bounding values. Bounds are only valid if the
+                        line is vertical or horizontal.
         hoverPen        Pen to use when drawing line when hovering over it. Can be any
                         arguments that are valid for :func:`mkPen <pyqtgraph.mkPen>`.
                         Default pen is red.
-        bounds          Optional [min, max] bounding values. Bounds are only valid if the
-                        line is vertical or horizontal.
         label           Text to be displayed in a label attached to the line, or
                         None to show no label (default is None). May optionally
                         include formatting strings to display the line value.
         labelOpts       A dict of keyword arguments to use when constructing the
                         text label. See :class:`InfLineLabel`.
+        span            Optional tuple (min, max) giving the range over the view to draw
+                        the line. For example, with a vertical line, use span=(0.5, 1)
+                        to draw only on the top half of the view.
+        markers         List of (marker, position, size) tuples, one per marker to display
+                        on the line. See the addMarker method.
         name            Name of the item
         =============== ==================================================================
         """
         self._boundingRect = None
-        self._line = None
 
         self._name = name
 
@@ -79,11 +90,25 @@ class InfiniteLine(GraphicsObject):
         if pen is None:
             pen = (200, 200, 100)
         self.setPen(pen)
+        
         if hoverPen is None:
             self.setHoverPen(color=(255,0,0), width=self.pen.width())
         else:
             self.setHoverPen(hoverPen)
+        
+        self.span = span
         self.currentPen = self.pen
+
+        self.markers = []
+        self._maxMarkerSize = 0
+        if markers is not None:
+            for m in markers:
+                self.addMarker(*m)
+                
+        # Cache variables for managing bounds
+        self._endPoints = [0, 1] # 
+        self._bounds = None
+        self._lastViewSize = None
         
         if label is not None:
             labelOpts = {} if labelOpts is None else labelOpts
@@ -98,7 +123,12 @@ class InfiniteLine(GraphicsObject):
         """Set the (minimum, maximum) allowable values when dragging."""
         self.maxRange = bounds
         self.setValue(self.value())
-
+        
+    def bounds(self):
+        """Return the (minimum, maximum) values allowed when dragging.
+        """
+        return self.maxRange[:]
+        
     def setPen(self, *args, **kwargs):
         """Set the pen for drawing the line. Allowable arguments are any that are valid
         for :func:`mkPen <pyqtgraph.mkPen>`."""
@@ -115,11 +145,71 @@ class InfiniteLine(GraphicsObject):
         If the line is not movable, then hovering is also disabled.
 
         Added in version 0.9.9."""
+        # If user did not supply a width, then copy it from pen
+        widthSpecified = ((len(args) == 1 and 
+                           (isinstance(args[0], QtGui.QPen) or
+                           (isinstance(args[0], dict) and 'width' in args[0]))
+                          ) or 'width' in kwargs)
         self.hoverPen = fn.mkPen(*args, **kwargs)
+        if not widthSpecified:
+            self.hoverPen.setWidth(self.pen.width())
+            
         if self.mouseHovering:
             self.currentPen = self.hoverPen
             self.update()
+        
+    def addMarker(self, marker, position=0.5, size=10.0):
+        """Add a marker to be displayed on the line. 
+        
+        ============= =========================================================
+        **Arguments**
+        marker        String indicating the style of marker to add:
+                      ``'<|'``, ``'|>'``, ``'>|'``, ``'|<'``, ``'<|>'``,
+                      ``'>|<'``, ``'^'``, ``'v'``, ``'o'``
+        position      Position (0.0-1.0) along the visible extent of the line
+                      to place the marker. Default is 0.5.
+        size          Size of the marker in pixels. Default is 10.0.
+        ============= =========================================================
+        """
+        path = QtGui.QPainterPath()
+        if marker == 'o': 
+            path.addEllipse(QtCore.QRectF(-0.5, -0.5, 1, 1))
+        if '<|' in marker:
+            p = QtGui.QPolygonF([Point(0.5, 0), Point(0, -0.5), Point(-0.5, 0)])
+            path.addPolygon(p)
+            path.closeSubpath()
+        if '|>' in marker:
+            p = QtGui.QPolygonF([Point(0.5, 0), Point(0, 0.5), Point(-0.5, 0)])
+            path.addPolygon(p)
+            path.closeSubpath()
+        if '>|' in marker:
+            p = QtGui.QPolygonF([Point(0.5, -0.5), Point(0, 0), Point(-0.5, -0.5)])
+            path.addPolygon(p)
+            path.closeSubpath()
+        if '|<' in marker:
+            p = QtGui.QPolygonF([Point(0.5, 0.5), Point(0, 0), Point(-0.5, 0.5)])
+            path.addPolygon(p)
+            path.closeSubpath()
+        if '^' in marker:
+            p = QtGui.QPolygonF([Point(0, -0.5), Point(0.5, 0), Point(0, 0.5)])
+            path.addPolygon(p)
+            path.closeSubpath()
+        if 'v' in marker:
+            p = QtGui.QPolygonF([Point(0, -0.5), Point(-0.5, 0), Point(0, 0.5)])
+            path.addPolygon(p)
+            path.closeSubpath()
+        
+        self.markers.append((path, position, size))
+        self._maxMarkerSize = max([m[2] / 2. for m in self.markers])
+        self.update()
 
+    def clearMarkers(self):
+        """ Remove all markers from this line.
+        """
+        self.markers = []
+        self._maxMarkerSize = 0
+        self.update()
+        
     def setAngle(self, angle):
         """
         Takes angle argument in degrees.
@@ -128,15 +218,15 @@ class InfiniteLine(GraphicsObject):
         Note that the use of value() and setValue() changes if the line is
         not vertical or horizontal.
         """
-        self.angle = ((angle+45) % 180) - 45   ##  -45 <= angle < 135
+        self.angle = angle #((angle+45) % 180) - 45   ##  -45 <= angle < 135
         self.resetTransform()
-        self.rotate(self.angle)
+        self.setRotation(self.angle)
         self.update()
 
     def setPos(self, pos):
 
-        if type(pos) in [list, tuple]:
-            newPos = pos
+        if isinstance(pos, (list, tuple, np.ndarray)) and not np.ndim(pos) == 0:
+            newPos = list(pos)
         elif isinstance(pos, QtCore.QPointF):
             newPos = [pos.x(), pos.y()]
         else:
@@ -161,7 +251,7 @@ class InfiniteLine(GraphicsObject):
 
         if self.p != newPos:
             self.p = newPos
-            self._invalidateCache()
+            self.viewTransformChanged()
             GraphicsObject.setPos(self, Point(self.p))
             self.sigPositionChanged.emit(self)
 
@@ -192,42 +282,102 @@ class InfiniteLine(GraphicsObject):
 
     ## broken in 4.7
     #def itemChange(self, change, val):
-        #if change in [self.ItemScenePositionHasChanged, self.ItemSceneHasChanged]:
+        #if change in [self.GraphicsItemChange.ItemScenePositionHasChanged, self.GraphicsItemChange.ItemSceneHasChanged]:
             #self.updateLine()
             #print "update", change
             #print self.getBoundingParents()
         #else:
             #print "ignore", change
         #return GraphicsObject.itemChange(self, change, val)
+    
+    def setSpan(self, mn, mx):
+        if self.span != (mn, mx):
+            self.span = (mn, mx)
+            self.update()
 
-    def _invalidateCache(self):
-        self._line = None
-        self._boundingRect = None
+    def _computeBoundingRect(self):
+        #br = UIGraphicsItem.boundingRect(self)
+        vr = self.viewRect()  # bounds of containing ViewBox mapped to local coords.
+        if vr is None:
+            return QtCore.QRectF()
+        
+        ## add a 4-pixel radius around the line for mouse interaction.
+        
+        px = self.pixelLength(direction=Point(1,0), ortho=True)  ## get pixel length orthogonal to the line
+        if px is None:
+            px = 0
+        pw = max(self.pen.width() / 2, self.hoverPen.width() / 2)
+        w = max(4, self._maxMarkerSize + pw) + 1
+        w = w * px
+        br = QtCore.QRectF(vr)
+        br.setBottom(-w)
+        br.setTop(w)
+
+        length = br.width()
+        left = br.left() + length * self.span[0]
+        right = br.left() + length * self.span[1]
+        br.setLeft(left)
+        br.setRight(right)
+        br = br.normalized()
+        
+        vs = self.getViewBox().size()
+        
+        if self._bounds != br or self._lastViewSize != vs:
+            self._bounds = br
+            self._lastViewSize = vs
+            self.prepareGeometryChange()
+        
+        self._endPoints = (left, right)
+        self._lastViewRect = vr
+        
+        return self._bounds
 
     def boundingRect(self):
         if self._boundingRect is None:
-            #br = UIGraphicsItem.boundingRect(self)
-            br = self.viewRect()
-            if br is None:
-                return QtCore.QRectF()
-            
-            ## add a 4-pixel radius around the line for mouse interaction.
-            px = self.pixelLength(direction=Point(1,0), ortho=True)  ## get pixel length orthogonal to the line
-            if px is None:
-                px = 0
-            w = (max(4, self.pen.width()/2, self.hoverPen.width()/2)+1) * px
-            br.setBottom(-w)
-            br.setTop(w)
-            
-            br = br.normalized()
-            self._boundingRect = br
-            self._line = QtCore.QLineF(br.right(), 0.0, br.left(), 0.0)
+            self._boundingRect = self._computeBoundingRect()
         return self._boundingRect
 
     def paint(self, p, *args):
-        p.setPen(self.currentPen)
-        p.drawLine(self._line)
-
+        p.setRenderHint(p.RenderHint.Antialiasing)
+        
+        left, right = self._endPoints
+        pen = self.currentPen
+        pen.setJoinStyle(QtCore.Qt.PenJoinStyle.MiterJoin)
+        p.setPen(pen)
+        p.drawLine(Point(left, 0), Point(right, 0))
+        
+        
+        if len(self.markers) == 0:
+            return
+        
+        # paint markers in native coordinate system
+        tr = p.transform()
+        p.resetTransform()
+        
+        start = tr.map(Point(left, 0))
+        end = tr.map(Point(right, 0))
+        up = tr.map(Point(left, 1))
+        dif = end - start
+        length = Point(dif).length()
+        angle = degrees(atan2(dif.y(), dif.x()))
+        
+        p.translate(start)
+        p.rotate(angle)
+        
+        up = up - start
+        det = up.x() * dif.y() - dif.x() * up.y()
+        p.scale(1, 1 if det > 0 else -1)
+        
+        p.setBrush(fn.mkBrush(self.currentPen.color()))
+        #p.setPen(fn.mkPen(None))
+        tr = p.transform()
+        for path, pos, size in self.markers:
+            p.setTransform(tr)
+            x = length * pos
+            p.translate(x, 0)
+            p.scale(size, size)
+            p.drawPath(path)
+        
     def dataBounds(self, axis, frac=1.0, orthoRange=None):
         if axis == 0:
             return None   ## x axis should never be auto-scaled
@@ -235,7 +385,7 @@ class InfiniteLine(GraphicsObject):
             return (0,0)
 
     def mouseDragEvent(self, ev):
-        if self.movable and ev.button() == QtCore.Qt.LeftButton:
+        if self.movable and ev.button() == QtCore.Qt.MouseButton.LeftButton:
             if ev.isStart():
                 self.moving = True
                 self.cursorOffset = self.pos() - self.mapToParent(ev.buttonDownPos())
@@ -252,7 +402,8 @@ class InfiniteLine(GraphicsObject):
                 self.sigPositionChangeFinished.emit(self)
 
     def mouseClickEvent(self, ev):
-        if self.moving and ev.button() == QtCore.Qt.RightButton:
+        self.sigClicked.emit(self, ev)
+        if self.moving and ev.button() == QtCore.Qt.MouseButton.RightButton:
             ev.accept()
             self.setPos(self.startPosition)
             self.moving = False
@@ -260,7 +411,7 @@ class InfiniteLine(GraphicsObject):
             self.sigPositionChangeFinished.emit(self)
 
     def hoverEvent(self, ev):
-        if (not ev.isExit()) and self.movable and ev.acceptDrags(QtCore.Qt.LeftButton):
+        if (not ev.isExit()) and self.movable and ev.acceptDrags(QtCore.Qt.MouseButton.LeftButton):
             self.setMouseHover(True)
         else:
             self.setMouseHover(False)
@@ -281,7 +432,8 @@ class InfiniteLine(GraphicsObject):
         Called whenever the transformation matrix of the view has changed.
         (eg, the view range has changed or the view was resized)
         """
-        self._invalidateCache()
+        self._boundingRect = None
+        GraphicsItem.viewTransformChanged(self)
         
     def setName(self, name):
         self._name = name
@@ -296,11 +448,11 @@ class InfLineLabel(TextItem):
     
     This class extends TextItem with the following features:
     
-    * Automatically positions adjacent to the line at a fixed position along
-      the line and within the view box.
-    * Automatically reformats text when the line value has changed.
-    * Can optionally be dragged to change its location along the line.
-    * Optionally aligns to its parent line.
+      * Automatically positions adjacent to the line at a fixed position along
+        the line and within the view box.
+      * Automatically reformats text when the line value has changed.
+      * Can optionally be dragged to change its location along the line.
+      * Optionally aligns to its parent line.
 
     =============== ==================================================================
     **Arguments:**
@@ -401,8 +553,7 @@ class InfLineLabel(TextItem):
     def setVisible(self, v):
         TextItem.setVisible(self, v)
         if v:
-            self.updateText()
-            self.updatePosition()
+            self.valueChanged()
             
     def setMovable(self, m):
         """Set whether this label is movable by dragging along the line.
@@ -430,7 +581,7 @@ class InfLineLabel(TextItem):
         self.valueChanged()
         
     def mouseDragEvent(self, ev):
-        if self.movable and ev.button() == QtCore.Qt.LeftButton:
+        if self.movable and ev.button() == QtCore.Qt.MouseButton.LeftButton:
             if ev.isStart():
                 self._moving = True
                 self._cursorOffset = self._posToRel(ev.buttonDownPos())
@@ -441,22 +592,23 @@ class InfLineLabel(TextItem):
                 return
 
             rel = self._posToRel(ev.pos())
-            self.orthoPos = np.clip(self._startPosition + rel - self._cursorOffset, 0, 1)
+            self.orthoPos = fn.clip_scalar(self._startPosition + rel - self._cursorOffset, 0., 1.)
             self.updatePosition()
             if ev.isFinish():
                 self._moving = False
 
     def mouseClickEvent(self, ev):
-        if self.moving and ev.button() == QtCore.Qt.RightButton:
+        if self.moving and ev.button() == QtCore.Qt.MouseButton.RightButton:
             ev.accept()
             self.orthoPos = self._startPosition
             self.moving = False
 
     def hoverEvent(self, ev):
         if not ev.isExit() and self.movable:
-            ev.acceptDrags(QtCore.Qt.LeftButton)
+            ev.acceptDrags(QtCore.Qt.MouseButton.LeftButton)
 
     def viewTransformChanged(self):
+        GraphicsItem.viewTransformChanged(self)
         self.updatePosition()
         TextItem.viewTransformChanged(self)
 
