@@ -1,32 +1,114 @@
-# -*- coding: utf-8 -*-
-from ..Qt import QtGui, QtCore
-try:
-    from ..Qt import QtOpenGL
-    HAVE_OPENGL = True
-except:
-    HAVE_OPENGL = False
+from ..Qt import QtCore, QtGui, QtWidgets
 
+HAVE_OPENGL = hasattr(QtWidgets, 'QOpenGLWidget')
+import itertools
+import math
+import sys
 import warnings
+
 import numpy as np
-from .GraphicsObject import GraphicsObject
+
+from .. import Qt, debug
 from .. import functions as fn
-from ..Point import Point
-import struct, sys
 from .. import getConfigOption
-from .. import debug
+from .GraphicsObject import GraphicsObject
 
 __all__ = ['PlotCurveItem']
+
+
+if Qt.QT_LIB.startswith('PyQt'):
+    wrapinstance = Qt.sip.wrapinstance
+else:
+    wrapinstance = Qt.shiboken.wrapInstance
+
+
+class LineSegments:
+    def __init__(self):
+        self.alloc(0)
+
+    def alloc(self, size):
+        self.arr = np.empty((size, 4), dtype=np.float64)
+        self.ptrs = list(map(wrapinstance,
+            itertools.count(self.arr.ctypes.data, self.arr.strides[0]),
+            itertools.repeat(QtCore.QLineF, self.arr.shape[0])))
+
+    def array(self, size):
+        if size > self.arr.shape[0]:
+            self.alloc(size + 16)
+        return self.arr[:size]
+
+    def instances(self, size):
+        return self.ptrs[:size]
+
+    def arrayToLineSegments(self, x, y, connect, finiteCheck):
+        # analogue of arrayToQPath taking the same parameters
+        if len(x) < 2:
+            return []
+
+        connect_array = None
+        if isinstance(connect, np.ndarray):
+            connect_array, connect = connect, 'array'
+
+        all_finite = True
+        if finiteCheck or connect == 'finite':
+            mask = np.isfinite(x) & np.isfinite(y)
+            all_finite = np.all(mask)
+
+        if connect == 'all':
+            if not all_finite:
+                # remove non-finite points, if any
+                x = x.compress(mask)
+                y = y.compress(mask)
+
+        elif connect == 'finite':
+            if not all_finite:
+                # each non-finite point affects the segment before and after
+                connect_array = mask[:-1] & mask[1:]
+
+        elif connect in ['pairs', 'array']:
+            if not all_finite:
+                # replicate the behavior of arrayToQPath
+                backfill_idx = fn._compute_backfill_indices(mask)
+                x = x[backfill_idx]
+                y = y[backfill_idx]
+
+        npts = len(x)
+        if npts < 2:
+            return []
+
+        segs = []
+
+        if connect in ['all', 'finite', 'array']:
+            memory = self.array(npts - 1)
+            memory[:, 0] = x[:-1]
+            memory[:, 1] = y[:-1]
+            memory[:, 2] = x[1:]
+            memory[:, 3] = y[1:]
+            segs = self.instances(npts - 1)
+            if connect_array is not None:
+                segs = list(itertools.compress(segs, connect_array.tolist()))
+
+        elif connect in ['pairs']:
+            npairs = npts // 2
+            memory = self.array(npairs).reshape((-1, 2))
+            memory[:, 0] = x[:npairs * 2]
+            memory[:, 1] = y[:npairs * 2]
+            segs = self.instances(npairs)
+
+        return segs
+
+
 class PlotCurveItem(GraphicsObject):
     """
     Class representing a single plot curve. Instances of this class are created
-    automatically as part of PlotDataItem; these rarely need to be instantiated
-    directly.
+    automatically as part of :class:`PlotDataItem <pyqtgraph.PlotDataItem>`; 
+    these rarely need to be instantiated directly.
 
     Features:
 
-    - Fast data update
-    - Fill under curve
-    - Mouse interaction
+      - Fast data update
+      - Fill under curve
+      - Mouse interaction
 
     =====================  ===============================================
     **Signals:**
@@ -47,15 +129,15 @@ class PlotCurveItem(GraphicsObject):
         ==============  =======================================================
         **Arguments:**
         parent          The parent GraphicsObject (optional)
-        clickable       If True, the item will emit sigClicked when it is
-                        clicked on. Defaults to False.
+        clickable       If `True`, the item will emit ``sigClicked`` when it is
+                        clicked on. Defaults to `False`.
         ==============  =======================================================
         """
         GraphicsObject.__init__(self, kargs.get('parent', None))
         self.clear()
 
         ## this is disastrous for performance.
-        #self.setCacheMode(QtGui.QGraphicsItem.DeviceCoordinateCache)
+        #self.setCacheMode(QtWidgets.QGraphicsItem.CacheMode.DeviceCoordinateCache)
 
         self.metaData = {}
         self.opts = {
@@ -69,6 +151,7 @@ class PlotCurveItem(GraphicsObject):
             'connect': 'all',
             'mouseWidth': 8, # width of shape responding to mouse click
             'compositionMode': None,
+            'skipFiniteCheck': False
         }
         if 'pen' not in kargs:
             self.opts['pen'] = fn.mkPen('w')
@@ -87,7 +170,7 @@ class PlotCurveItem(GraphicsObject):
     def setClickable(self, s, width=None):
         """Sets whether the item responds to mouse clicks.
 
-        The *width* argument specifies the width in pixels orthogonal to the
+        The `width` argument specifies the width in pixels orthogonal to the
         curve that will respond to a mouse click.
         """
         self.clickable = s
@@ -97,20 +180,32 @@ class PlotCurveItem(GraphicsObject):
             self._boundingRect = None
 
     def setCompositionMode(self, mode):
-        """Change the composition mode of the item (see QPainter::CompositionMode
-        in the Qt documentation). This is useful when overlaying multiple items.
+        """
+        Change the composition mode of the item. This is useful when overlaying
+        multiple items.
+        
+        Parameters
+        ----------
+        mode : ``QtGui.QPainter.CompositionMode``
+            Composition of the item, often used when overlaying items.  Common
+            options include:
 
-        ============================================  ============================================================
-        **Most common arguments:**
-        QtGui.QPainter.CompositionMode_SourceOver     Default; image replaces the background if it
-                                                      is opaque. Otherwise, it uses the alpha channel to blend
-                                                      the image with the background.
-        QtGui.QPainter.CompositionMode_Overlay        The image color is mixed with the background color to
-                                                      reflect the lightness or darkness of the background.
-        QtGui.QPainter.CompositionMode_Plus           Both the alpha and color of the image and background pixels
-                                                      are added together.
-        QtGui.QPainter.CompositionMode_Multiply       The output is the image color multiplied by the background.
-        ============================================  ============================================================
+            ``QPainter.CompositionMode.CompositionMode_SourceOver`` (Default)
+            Image replaces the background if it is opaque. Otherwise, it uses
+            the alpha channel to blend the image with the background.
+
+            ``QPainter.CompositionMode.CompositionMode_Overlay`` Image color is
+            mixed with the background color to reflect the lightness or
+            darkness of the background
+
+            ``QPainter.CompositionMode.CompositionMode_Plus`` Both the alpha
+            and color of the image and background pixels are added together.
+
+            ``QPainter.CompositionMode.CompositionMode_Plus`` The output is the
+            image color multiplied by the background.
+
+            See ``QPainter::CompositionMode`` in the Qt Documentation for more
+            options and details
         """
         self.opts['compositionMode'] = mode
         self.update()
@@ -135,6 +230,8 @@ class PlotCurveItem(GraphicsObject):
         elif ax == 1:
             d = y
             d2 = x
+        else:
+            raise ValueError("Invalid axis value")
 
         ## If an orthogonal range is specified, mask the data now
         if orthoRange is not None:
@@ -153,7 +250,7 @@ class PlotCurveItem(GraphicsObject):
                 # All-NaN data is acceptable; Explicit numpy warning is not needed.
                 warnings.simplefilter("ignore")
                 b = (np.nanmin(d), np.nanmax(d))
-            if any(np.isinf(b)):
+            if math.isinf(b[0]) or math.isinf(b[1]):
                 mask = np.isfinite(d)
                 d = d[mask]
                 if len(d) == 0:
@@ -179,7 +276,7 @@ class PlotCurveItem(GraphicsObject):
         spen = self.opts['shadowPen']
         if not pen.isCosmetic():
             b = (b[0] - pen.widthF()*0.7072, b[1] + pen.widthF()*0.7072)
-        if spen is not None and not spen.isCosmetic() and spen.style() != QtCore.Qt.NoPen:
+        if spen is not None and not spen.isCosmetic() and spen.style() != QtCore.Qt.PenStyle.NoPen:
             b = (b[0] - spen.widthF()*0.7072, b[1] + spen.widthF()*0.7072)
 
         self._boundsCache[ax] = [(frac, orthoRange), b]
@@ -191,7 +288,7 @@ class PlotCurveItem(GraphicsObject):
         w = 0
         if pen.isCosmetic():
             w += pen.widthF()*0.7072
-        if spen is not None and spen.isCosmetic() and spen.style() != QtCore.Qt.NoPen:
+        if spen is not None and spen.isCosmetic() and spen.style() != QtCore.Qt.PenStyle.NoPen:
             w = max(w, spen.widthF()*0.7072)
         if self.clickable:
             w = max(w, self.opts['mouseWidth']//2 + 1)
@@ -274,77 +371,110 @@ class PlotCurveItem(GraphicsObject):
 
     def setPen(self, *args, **kargs):
         """Set the pen used to draw the curve."""
-        self.opts['pen'] = fn.mkPen(*args, **kargs)
+        if args[0] is None:
+            self.opts['pen'] = None
+        else:
+            self.opts['pen'] = fn.mkPen(*args, **kargs)
         self.invalidateBounds()
         self.update()
 
     def setShadowPen(self, *args, **kargs):
-        """Set the shadow pen used to draw behind the primary pen.
-        This pen must have a larger width than the primary
-        pen to be visible.
         """
-        self.opts['shadowPen'] = fn.mkPen(*args, **kargs)
+        Set the shadow pen used to draw behind the primary pen.
+        This pen must have a larger width than the primary
+        pen to be visible. Arguments are passed to 
+        :func:`mkPen <pyqtgraph.mkPen>`
+        """
+        if args[0] is None:
+            self.opts['shadowPen'] = None
+        else:
+            self.opts['shadowPen'] = fn.mkPen(*args, **kargs)
         self.invalidateBounds()
         self.update()
 
     def setBrush(self, *args, **kargs):
-        """Set the brush used when filling the area under the curve"""
-        self.opts['brush'] = fn.mkBrush(*args, **kargs)
+        """
+        Sets the brush used when filling the area under the curve. All 
+        arguments are passed to :func:`mkBrush <pyqtgraph.mkBrush>`.
+        """
+        if args[0] is None:
+            self.opts['brush'] = None
+        else:
+            self.opts['brush'] = fn.mkBrush(*args, **kargs)
         self.invalidateBounds()
         self.update()
 
     def setFillLevel(self, level):
-        """Set the level filled to when filling under the curve"""
+        """Sets the level filled to when filling under the curve"""
         self.opts['fillLevel'] = level
         self.fillPath = None
+        self._fillPathList = None
         self.invalidateBounds()
         self.update()
+        
+    def setSkipFiniteCheck(self, skipFiniteCheck):
+        """
+        When it is known that the plot data passed to ``PlotCurveItem`` contains only finite numerical values,
+        the `skipFiniteCheck` property can help speed up plotting. If this flag is set and the data contains 
+        any non-finite values (such as `NaN` or `Inf`), unpredictable behavior will occur. The data might not
+        be plotted, or there migth be significant performance impact.
+        """
+        self.opts['skipFiniteCheck']  = bool(skipFiniteCheck)
 
     def setData(self, *args, **kargs):
         """
-        =============== ========================================================
+        =============== =================================================================
         **Arguments:**
-        x, y            (numpy arrays) Data to show
+        x, y            (numpy arrays) Data to display
         pen             Pen to use when drawing. Any single argument accepted by
                         :func:`mkPen <pyqtgraph.mkPen>` is allowed.
         shadowPen       Pen for drawing behind the primary pen. Usually this
                         is used to emphasize the curve by providing a
                         high-contrast border. Any single argument accepted by
                         :func:`mkPen <pyqtgraph.mkPen>` is allowed.
-        fillLevel       (float or None) Fill the area 'under' the curve to
-                        *fillLevel*
-        fillOutline     (bool) If True, an outline surrounding the *fillLevel*
+        fillLevel       (float or None) Fill the area under the curve to
+                        the specified value.
+        fillOutline     (bool) If True, an outline surrounding the `fillLevel`
                         area is drawn.
-        brush           QBrush to use when filling. Any single argument accepted
+        brush           Brush to use when filling. Any single argument accepted
                         by :func:`mkBrush <pyqtgraph.mkBrush>` is allowed.
         antialias       (bool) Whether to use antialiasing when drawing. This
                         is disabled by default because it decreases performance.
-        stepMode        (str or None) If "center", a step is drawn using the x
-                        values as boundaries and the given y values are
+        stepMode        (str or None) If 'center', a step is drawn using the `x`
+                        values as boundaries and the given `y` values are
                         associated to the mid-points between the boundaries of
                         each step. This is commonly used when drawing
-                        histograms. Note that in this case, len(x) == len(y) + 1
-                        If "left" or "right", the step is drawn assuming that
-                        the y value is associated to the left or right boundary,
-                        respectively. In this case len(x) == len(y)
-                        If not passed or an empty string or None is passed, the
+                        histograms. Note that in this case, ``len(x) == len(y) + 1``
+                        
+                        If 'left' or 'right', the step is drawn assuming that
+                        the `y` value is associated to the left or right boundary,
+                        respectively. In this case ``len(x) == len(y)``
+                        If not passed or an empty string or `None` is passed, the
                         step mode is not enabled.
-                        Passing True is a deprecated equivalent to "center".
         connect         Argument specifying how vertexes should be connected
-                        by line segments. Default is "all", indicating full
-                        connection. "pairs" causes only even-numbered segments
-                        to be drawn. "finite" causes segments to be omitted if
-                        they are attached to nan or inf values. For any other
-                        connectivity, specify an array of boolean values.
+                        by line segments. 
+                        
+                            | 'all' (default) indicates full connection. 
+                            | 'pairs' draws one separate line segment for each two points given.
+                            | 'finite' omits segments attached to `NaN` or `Inf` values. 
+                            | For any other connectivity, specify an array of boolean values.
         compositionMode See :func:`setCompositionMode
                         <pyqtgraph.PlotCurveItem.setCompositionMode>`.
-        =============== ========================================================
+        skipFiniteCheck (bool, defaults to `False`) Optimization flag that can
+                        speed up plotting by not checking and compensating for
+                        `NaN` values.  If set to `True`, and `NaN` values exist, the
+                        data may not be displayed or the plot may take a
+                        significant performance hit.
+        =============== =================================================================
 
         If non-keyword arguments are used, they will be interpreted as
-        setData(y) for a single argument and setData(x, y) for two
+        ``setData(y)`` for a single argument and ``setData(x, y)`` for two
         arguments.
-
-
+        
+        **Notes on performance:**
+        
+        Line widths greater than 1 pixel affect the performance as discussed in 
+        the documentation of :class:`PlotDataItem <pyqtgraph.PlotDataItem>`.
         """
         self.updateData(*args, **kargs)
 
@@ -375,9 +505,10 @@ class PlotCurveItem(GraphicsObject):
             if data.dtype.kind == 'c':
                 raise Exception("Can not plot complex data types.")
 
+
         profiler("data checks")
 
-        #self.setCacheMode(QtGui.QGraphicsItem.NoCache)  ## Disabling and re-enabling the cache works around a bug in Qt 4.6 causing the cached results to display incorrectly
+        #self.setCacheMode(QtWidgets.QGraphicsItem.CacheMode.NoCache)  ## Disabling and re-enabling the cache works around a bug in Qt 4.6 causing the cached results to display incorrectly
                                                         ##    Test this bug with test_PlotWidget and zoom in on the animated plot
         self.yData = kargs['y'].view(np.ndarray)
         self.xData = kargs['x'].view(np.ndarray)
@@ -393,8 +524,10 @@ class PlotCurveItem(GraphicsObject):
 
         if self.opts['stepMode'] in ("center", True):  ## check against True for backwards compatibility
             if self.opts['stepMode'] is True:
-                import warnings
-                warnings.warn('stepMode=True is deprecated, use stepMode="center" instead', DeprecationWarning, stacklevel=3)
+                warnings.warn(
+                    'stepMode=True is deprecated and will result in an error after October 2022. Use stepMode="center" instead.',
+                    DeprecationWarning, stacklevel=3
+                )
             if len(self.xData) != len(self.yData)+1:  ## allow difference of 1 for step mode plots
                 raise Exception("len(X) must be len(Y)+1 since stepMode=True (got %s and %s)" % (self.xData.shape, self.yData.shape))
         else:
@@ -403,8 +536,9 @@ class PlotCurveItem(GraphicsObject):
 
         self.path = None
         self.fillPath = None
+        self._fillPathList = None
         self._mouseShape = None
-        #self.xDisp = self.yDisp = None
+        self._renderSegmentList = None
 
         if 'name' in kargs:
             self.opts['name'] = kargs['name']
@@ -412,16 +546,18 @@ class PlotCurveItem(GraphicsObject):
             self.opts['connect'] = kargs['connect']
         if 'pen' in kargs:
             self.setPen(kargs['pen'])
-        if 'shadowPen' in kargs and kargs['shadowPen'] is not None:
+        if 'shadowPen' in kargs:
             self.setShadowPen(kargs['shadowPen'])
-        if 'fillLevel' in kargs and kargs['fillLevel'] is not None:
+        if 'fillLevel' in kargs:
             self.setFillLevel(kargs['fillLevel'])
         if 'fillOutline' in kargs:
             self.opts['fillOutline'] = kargs['fillOutline']
-        if 'brush' in kargs and kargs['brush'] is not None:
+        if 'brush' in kargs:
             self.setBrush(kargs['brush'])
         if 'antialias' in kargs:
             self.opts['antialias'] = kargs['antialias']
+        if 'skipFiniteCheck' in kargs:
+            self.opts['skipFiniteCheck'] = kargs['skipFiniteCheck']
 
         profiler('set')
         self.update()
@@ -429,41 +565,51 @@ class PlotCurveItem(GraphicsObject):
         self.sigPlotChanged.emit(self)
         profiler('emit')
 
+    @staticmethod
+    def _generateStepModeData(stepMode, x, y, baseline):
+        ## each value in the x/y arrays generates 2 points.
+        if stepMode == "right":
+            x2 = np.empty((len(x) + 1, 2), dtype=x.dtype)
+            x2[:-1] = x[:, np.newaxis]
+            x2[-1] = x2[-2]
+        elif stepMode == "left":
+            x2 = np.empty((len(x) + 1, 2), dtype=x.dtype)
+            x2[1:] = x[:, np.newaxis]
+            x2[0] = x2[1]
+        elif stepMode in ("center", True):  ## support True for back-compat
+            x2 = np.empty((len(x),2), dtype=x.dtype)
+            x2[:] = x[:, np.newaxis]
+        else:
+            raise ValueError("Unsupported stepMode %s" % stepMode)
+        if baseline is None:
+            x = x2.reshape(x2.size)[1:-1]
+            y2 = np.empty((len(y),2), dtype=y.dtype)
+            y2[:] = y[:,np.newaxis]
+            y = y2.reshape(y2.size)
+        else:
+            # if baseline is provided, add vertical lines to left/right ends
+            x = x2.reshape(x2.size)
+            y2 = np.empty((len(y)+2,2), dtype=y.dtype)
+            y2[1:-1] = y[:,np.newaxis]
+            y = y2.reshape(y2.size)[1:-1]
+            y[[0, -1]] = baseline
+        return x, y
+
     def generatePath(self, x, y):
-        stepMode = self.opts['stepMode']
-        if stepMode:
-            ## each value in the x/y arrays generates 2 points.
-            if stepMode == "right":
-                x2 = np.empty((len(x) + 1, 2), dtype=x.dtype)
-                x2[:-1] = x[:, np.newaxis]
-                x2[-1] = x2[-2]
-            elif stepMode == "left":
-                x2 = np.empty((len(x) + 1, 2), dtype=x.dtype)
-                x2[1:] = x[:, np.newaxis]
-                x2[0] = x2[1]
-            elif stepMode in ("center", True):  ## support True for back-compat
-                x2 = np.empty((len(x),2), dtype=x.dtype)
-                x2[:] = x[:, np.newaxis]
-            else:
-                raise ValueError("Unsupported stepMode %s" % stepMode)
-            if self.opts['fillLevel'] is None:
-                x = x2.reshape(x2.size)[1:-1]
-                y2 = np.empty((len(y),2), dtype=y.dtype)
-                y2[:] = y[:,np.newaxis]
-                y = y2.reshape(y2.size)
-            else:
-                ## If we have a fill level, add two extra points at either end
-                x = x2.reshape(x2.size)
-                y2 = np.empty((len(y)+2,2), dtype=y.dtype)
-                y2[1:-1] = y[:,np.newaxis]
-                y = y2.reshape(y2.size)[1:-1]
-                y[0] = self.opts['fillLevel']
-                y[-1] = self.opts['fillLevel']
-
-        path = fn.arrayToQPath(x, y, connect=self.opts['connect'])
-
-        return path
+        if self.opts['stepMode']:
+            x, y = self._generateStepModeData(
+                self.opts['stepMode'],
+                x,
+                y,
+                baseline=self.opts['fillLevel']
+            )
 
+        return fn.arrayToQPath(
+            x,
+            y,
+            connect=self.opts['connect'],
+            finiteCheck=not self.opts['skipFiniteCheck']
+        )
 
     def getPath(self):
         if self.path is None:
@@ -473,50 +619,178 @@ class PlotCurveItem(GraphicsObject):
             else:
                 self.path = self.generatePath(*self.getData())
             self.fillPath = None
+            self._fillPathList = None
             self._mouseShape = None
 
         return self.path
 
+    def _shouldUseDrawLineSegments(self, pen):
+        return (
+            pen.widthF() > 1.0
+            # non-solid pen styles need single polyline to be effective
+            and pen.style() == QtCore.Qt.PenStyle.SolidLine
+            # segmenting the curve slows gradient brushes, and is expected
+            # to do the same for other patterns
+            and pen.isSolid()   # pen.brush().style() == Qt.BrushStyle.SolidPattern
+            # ends of adjacent line segments overlapping is visible when not opaque
+            and pen.color().alphaF() == 1.0
+        )
+
+    def _getLineSegments(self):
+        if not hasattr(self, '_lineSegments'):
+            self._lineSegments = LineSegments()
+
+        if self._renderSegmentList is None:
+            x, y = self.getData()
+            if self.opts['stepMode']:
+                x, y = self._generateStepModeData(
+                    self.opts['stepMode'],
+                    x,
+                    y,
+                    baseline=self.opts['fillLevel']
+                )
+
+            self._renderSegmentList = self._lineSegments.arrayToLineSegments(
+                x,
+                y,
+                connect=self.opts['connect'],
+                finiteCheck=not self.opts['skipFiniteCheck']
+            )
+
+        return self._renderSegmentList
+
+    def _getClosingSegments(self):
+        # this is only used for fillOutline
+        # no point caching with so few elements generated
+        segments = []
+        if self.opts['fillLevel'] == 'enclosed':
+            return segments
+
+        baseline = self.opts['fillLevel']
+        x, y = self.getData()
+        lx, rx = x[[0, -1]]
+        ly, ry = y[[0, -1]]
+
+        if ry != baseline:
+            segments.append(QtCore.QLineF(rx, ry, rx, baseline))
+        segments.append(QtCore.QLineF(rx, baseline, lx, baseline))
+        if ly != baseline:
+            segments.append(QtCore.QLineF(lx, baseline, lx, ly))
+
+        return segments
+
+    def _getFillPath(self):
+        if self.fillPath is not None:
+            return self.fillPath
+
+        path = QtGui.QPainterPath(self.getPath())
+        self.fillPath = path
+        if self.opts['fillLevel'] == 'enclosed':
+            return path
+
+        baseline = self.opts['fillLevel']
+        x, y = self.getData()
+        lx, rx = x[[0, -1]]
+        ly, ry = y[[0, -1]]
+
+        if ry != baseline:
+            path.lineTo(rx, baseline)
+        path.lineTo(lx, baseline)
+        if ly != baseline:
+            path.lineTo(lx, ly)
+
+        return path
+
+    def _shouldUseFillPathList(self):
+        connect = self.opts['connect']
+        return (
+            # not meaningful to fill disjoint lines
+            isinstance(connect, str) and connect == 'all'
+            # guard against odd-ball argument 'enclosed'
+            and isinstance(self.opts['fillLevel'], (int, float))
+        )
+
+    def _getFillPathList(self):
+        if self._fillPathList is not None:
+            return self._fillPathList
+
+        x, y = self.getData()
+        if self.opts['stepMode']:
+            x, y = self._generateStepModeData(
+                self.opts['stepMode'],
+                x,
+                y,
+                # note that left/right vertical lines can be omitted here
+                baseline=None
+            )
+
+        if not self.opts['skipFiniteCheck']:
+            mask = np.isfinite(x) & np.isfinite(y)
+            if not mask.all():
+                # we are only supporting connect='all',
+                # so remove non-finite values
+                x = x.compress(mask)
+                y = y.compress(mask)
+
+        if len(x) < 2:
+            return []
+
+        paths = self._fillPathList = []
+        offset = 0
+        chunksize = 50          # determined empirically
+        xybuf = np.empty((chunksize+3, 2))
+        baseline = self.opts['fillLevel']
+
+        while offset < len(x) - 1:
+            subx = x[offset:offset + chunksize]
+            suby = y[offset:offset + chunksize]
+            size = len(subx)
+            xyview = xybuf[:size+3]
+            xyview[:-3, 0] = subx
+            xyview[:-3, 1] = suby
+            xyview[-3:, 0] = subx[[-1, 0, 0]]
+            xyview[-3:, 1] = [baseline, baseline, suby[0]]
+            offset += size - 1  # last point is re-used for next chunk
+            # data was either declared to be all-finite OR was sanitized
+            path = fn._arrayToQPath_all(xyview[:, 0], xyview[:, 1], finiteCheck=False)
+            paths.append(path)
+
+        return paths
+
     @debug.warnOnException  ## raising an exception here causes crash
     def paint(self, p, opt, widget):
         profiler = debug.Profiler()
         if self.xData is None or len(self.xData) == 0:
             return
 
-        if HAVE_OPENGL and getConfigOption('enableExperimental') and isinstance(widget, QtOpenGL.QGLWidget):
-            self.paintGL(p, opt, widget)
-            return
-
-        x = None
-        y = None
-        path = self.getPath()
-        profiler('generate path')
+        if getConfigOption('enableExperimental'):
+            if HAVE_OPENGL and isinstance(widget, QtWidgets.QOpenGLWidget):
+                self.paintGL(p, opt, widget)
+                return
 
         if self._exportOpts is not False:
             aa = self._exportOpts.get('antialias', True)
         else:
             aa = self.opts['antialias']
 
-        p.setRenderHint(p.Antialiasing, aa)
+        p.setRenderHint(p.RenderHint.Antialiasing, aa)
 
         cmode = self.opts['compositionMode']
         if cmode is not None:
             p.setCompositionMode(cmode)
 
-        if self.opts['brush'] is not None and self.opts['fillLevel'] is not None:
-            if self.fillPath is None:
-                if x is None:
-                    x,y = self.getData()
-                p2 = QtGui.QPainterPath(self.path)
-                if self.opts['fillLevel'] != 'enclosed':
-                    p2.lineTo(x[-1], self.opts['fillLevel'])
-                    p2.lineTo(x[0], self.opts['fillLevel'])
-                    p2.lineTo(x[0], y[0])
-                p2.closeSubpath()
-                self.fillPath = p2
+        do_fill = self.opts['brush'] is not None and self.opts['fillLevel'] is not None
+        do_fill_outline = do_fill and self.opts['fillOutline']
+
+        if do_fill:
+            if self._shouldUseFillPathList():
+                paths = self._getFillPathList()
+            else:
+                paths = [self._getFillPath()]
 
             profiler('generate fill path')
-            p.fillPath(self.fillPath, self.opts['brush'])
+            for path in paths:
+                p.fillPath(path, self.opts['brush'])
             profiler('draw fill path')
 
         # Avoid constructing a shadow pen if it's not used.
@@ -526,26 +800,52 @@ class PlotCurveItem(GraphicsObject):
             else:
                 sp = fn.mkPen(self.opts['shadowPen'])
 
-            if sp.style() != QtCore.Qt.NoPen:
+            if sp.style() != QtCore.Qt.PenStyle.NoPen:
                 p.setPen(sp)
-                p.drawPath(path)
+                if self._shouldUseDrawLineSegments(sp):
+                    p.drawLines(self._getLineSegments())
+                    if do_fill_outline:
+                        p.drawLines(self._getClosingSegments())
+                else:
+                    if do_fill_outline:
+                        p.drawPath(self._getFillPath())
+                    else:
+                        p.drawPath(self.getPath())
 
-        if isinstance(self.opts.get('pen'), QtGui.QPen):
-            cp = self.opts['pen']
-        else:
-            cp = fn.mkPen(self.opts['pen'])
+        cp = self.opts['pen']
+        if not isinstance(cp, QtGui.QPen):
+            cp = fn.mkPen(cp)
 
         p.setPen(cp)
-        if self.opts['fillOutline'] and self.fillPath is not None:
-            p.drawPath(self.fillPath)
+        if self._shouldUseDrawLineSegments(cp):
+            p.drawLines(self._getLineSegments())
+            if do_fill_outline:
+                p.drawLines(self._getClosingSegments())
         else:
-            p.drawPath(path)
+            if do_fill_outline:
+                p.drawPath(self._getFillPath())
+            else:
+                p.drawPath(self.getPath())
         profiler('drawPath')
 
     def paintGL(self, p, opt, widget):
         p.beginNativePainting()
         import OpenGL.GL as gl
 
+        if sys.platform == 'win32':
+            # If Qt is built to dynamically load OpenGL, then the projection and
+            # modelview matrices are not setup.
+            # https://doc.qt.io/qt-6/windows-graphics.html
+            # https://code.woboq.org/qt6/qtbase/src/opengl/qopenglpaintengine.cpp.html
+            # Technically, we could enable it for all platforms, but for now, just
+            # enable it where it is required, i.e. Windows
+            gl.glMatrixMode(gl.GL_PROJECTION)
+            gl.glLoadIdentity()
+            gl.glOrtho(0, widget.width(), widget.height(), 0, -999999, 999999)
+            gl.glMatrixMode(gl.GL_MODELVIEW)
+            mat = QtGui.QMatrix4x4(self.sceneTransform())
+            gl.glLoadMatrixf(np.array(mat.data(), dtype=np.float32))
+
         ## set clipping viewport
         view = self.getViewBox()
         if view is not None:
@@ -579,25 +879,34 @@ class PlotCurveItem(GraphicsObject):
 
         try:
             x, y = self.getData()
-            pos = np.empty((len(x), 2))
+            pos = np.empty((len(x), 2), dtype=np.float32)
             pos[:,0] = x
             pos[:,1] = y
             gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
             try:
                 gl.glVertexPointerf(pos)
                 pen = fn.mkPen(self.opts['pen'])
-                color = pen.color()
-                gl.glColor4f(color.red()/255., color.green()/255., color.blue()/255., color.alpha()/255.)
+                gl.glColor4f(*pen.color().getRgbF())
                 width = pen.width()
                 if pen.isCosmetic() and width < 1:
                     width = 1
                 gl.glPointSize(width)
                 gl.glLineWidth(width)
-                gl.glEnable(gl.GL_LINE_SMOOTH)
-                gl.glEnable(gl.GL_BLEND)
-                gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
-                gl.glHint(gl.GL_LINE_SMOOTH_HINT, gl.GL_NICEST)
-                gl.glDrawArrays(gl.GL_LINE_STRIP, 0, int(pos.size / pos.shape[-1]))
+
+                # enable antialiasing if requested
+                if self._exportOpts is not False:
+                    aa = self._exportOpts.get('antialias', True)
+                else:
+                    aa = self.opts['antialias']
+                if aa:
+                    gl.glEnable(gl.GL_LINE_SMOOTH)
+                    gl.glEnable(gl.GL_BLEND)
+                    gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
+                    gl.glHint(gl.GL_LINE_SMOOTH_HINT, gl.GL_NICEST)
+                else:
+                    gl.glDisable(gl.GL_LINE_SMOOTH)
+
+                gl.glDrawArrays(gl.GL_LINE_STRIP, 0, pos.shape[0])
             finally:
                 gl.glDisableClientState(gl.GL_VERTEX_ARRAY)
         finally:
@@ -606,10 +915,10 @@ class PlotCurveItem(GraphicsObject):
     def clear(self):
         self.xData = None  ## raw values
         self.yData = None
-        self.xDisp = None  ## display values (after log / fft)
-        self.yDisp = None
+        self._renderSegmentList = None
         self.path = None
         self.fillPath = None
+        self._fillPathList = None
         self._mouseShape = None
         self._mouseBounds = None
         self._boundsCache = [None, None]
@@ -633,7 +942,7 @@ class PlotCurveItem(GraphicsObject):
         return self._mouseShape
 
     def mouseClickEvent(self, ev):
-        if not self.clickable or ev.button() != QtCore.Qt.LeftButton:
+        if not self.clickable or ev.button() != QtCore.Qt.MouseButton.LeftButton:
             return
         if self.mouseShape().contains(ev.pos()):
             ev.accept()
