@@ -1,28 +1,27 @@
-# -*- coding: utf-8 -*-
+import collections.abc
 import importlib
 import os
 import warnings
 import weakref
-import collections.abc
 
 import numpy as np
 
+from ... import functions as fn
+from ... import icons
+from ...Qt import QT_LIB, QtCore, QtGui, QtWidgets
+from ...WidgetGroup import WidgetGroup
+from ...widgets.FileDialog import FileDialog
 from ..AxisItem import AxisItem
 from ..ButtonItem import ButtonItem
 from ..GraphicsWidget import GraphicsWidget
 from ..InfiniteLine import InfiniteLine
 from ..LabelItem import LabelItem
 from ..LegendItem import LegendItem
-from ..PlotDataItem import PlotDataItem
 from ..PlotCurveItem import PlotCurveItem
+from ..PlotDataItem import PlotDataItem
 from ..ScatterPlotItem import ScatterPlotItem
 from ..ViewBox import ViewBox
-from ... import functions as fn
-from ... import icons
-from ...Qt import QtGui, QtCore, QT_LIB
-from ...WidgetGroup import WidgetGroup
-from ...python2_3 import basestring
-from ...widgets.FileDialog import FileDialog
+
 
 translate = QtCore.QCoreApplication.translate
 
@@ -43,35 +42,35 @@ class PlotItem(GraphicsWidget):
 
     It's main functionality is:
 
-    - Manage placement of ViewBox, AxisItems, and LabelItems
-    - Create and manage a list of PlotDataItems displayed inside the ViewBox
-    - Implement a context menu with commonly used display and analysis options
+      - Manage placement of ViewBox, AxisItems, and LabelItems
+      - Create and manage a list of PlotDataItems displayed inside the ViewBox
+      - Implement a context menu with commonly used display and analysis options
 
     Use :func:`plot() <pyqtgraph.PlotItem.plot>` to create a new PlotDataItem and
     add it to the view. Use :func:`addItem() <pyqtgraph.PlotItem.addItem>` to
     add any QGraphicsItem to the view.
     
     This class wraps several methods from its internal ViewBox:
-    :func:`setXRange <pyqtgraph.ViewBox.setXRange>`,
-    :func:`setYRange <pyqtgraph.ViewBox.setYRange>`,
-    :func:`setRange <pyqtgraph.ViewBox.setRange>`,
-    :func:`autoRange <pyqtgraph.ViewBox.autoRange>`,
-    :func:`setDefaultPadding <pyqtgraph.ViewBox.setDefaultPadding>`,
-    :func:`setXLink <pyqtgraph.ViewBox.setXLink>`,
-    :func:`setYLink <pyqtgraph.ViewBox.setYLink>`,
-    :func:`setAutoPan <pyqtgraph.ViewBox.setAutoPan>`,
-    :func:`setAutoVisible <pyqtgraph.ViewBox.setAutoVisible>`,
-    :func:`setLimits <pyqtgraph.ViewBox.setLimits>`,
-    :func:`viewRect <pyqtgraph.ViewBox.viewRect>`,
-    :func:`viewRange <pyqtgraph.ViewBox.viewRange>`,
-    :func:`setMouseEnabled <pyqtgraph.ViewBox.setMouseEnabled>`,
-    :func:`enableAutoRange <pyqtgraph.ViewBox.enableAutoRange>`,
-    :func:`disableAutoRange <pyqtgraph.ViewBox.disableAutoRange>`,
-    :func:`setAspectLocked <pyqtgraph.ViewBox.setAspectLocked>`,
-    :func:`invertY <pyqtgraph.ViewBox.invertY>`,
-    :func:`invertX <pyqtgraph.ViewBox.invertX>`,
-    :func:`register <pyqtgraph.ViewBox.register>`,
-    :func:`unregister <pyqtgraph.ViewBox.unregister>`
+      - :func:`setXRange <pyqtgraph.ViewBox.setXRange>`
+      - :func:`setYRange <pyqtgraph.ViewBox.setYRange>`
+      - :func:`setRange <pyqtgraph.ViewBox.setRange>`
+      - :func:`autoRange <pyqtgraph.ViewBox.autoRange>`
+      - :func:`setDefaultPadding <pyqtgraph.ViewBox.setDefaultPadding>`
+      - :func:`setXLink <pyqtgraph.ViewBox.setXLink>`
+      - :func:`setYLink <pyqtgraph.ViewBox.setYLink>`
+      - :func:`setAutoPan <pyqtgraph.ViewBox.setAutoPan>`
+      - :func:`setAutoVisible <pyqtgraph.ViewBox.setAutoVisible>`
+      - :func:`setLimits <pyqtgraph.ViewBox.setLimits>`
+      - :func:`viewRect <pyqtgraph.ViewBox.viewRect>`
+      - :func:`viewRange <pyqtgraph.ViewBox.viewRange>`
+      - :func:`setMouseEnabled <pyqtgraph.ViewBox.setMouseEnabled>`
+      - :func:`enableAutoRange <pyqtgraph.ViewBox.enableAutoRange>`
+      - :func:`disableAutoRange <pyqtgraph.ViewBox.disableAutoRange>`
+      - :func:`setAspectLocked <pyqtgraph.ViewBox.setAspectLocked>`
+      - :func:`invertY <pyqtgraph.ViewBox.invertY>`
+      - :func:`invertX <pyqtgraph.ViewBox.invertX>`
+      - :func:`register <pyqtgraph.ViewBox.register>`
+      - :func:`unregister <pyqtgraph.ViewBox.unregister>`
     
     The ViewBox itself can be accessed by calling :func:`getViewBox() <pyqtgraph.PlotItem.getViewBox>` 
     
@@ -115,7 +114,7 @@ class PlotItem(GraphicsWidget):
         
         GraphicsWidget.__init__(self, parent)
         
-        self.setSizePolicy(QtGui.QSizePolicy.Policy.Expanding, QtGui.QSizePolicy.Policy.Expanding)
+        self.setSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
         
         ## Set up control buttons
         path = os.path.dirname(__file__)
@@ -125,7 +124,7 @@ class PlotItem(GraphicsWidget):
         self.buttonsHidden = False ## whether the user has requested buttons to be hidden
         self.mouseHovering = False
         
-        self.layout = QtGui.QGraphicsGridLayout()
+        self.layout = QtWidgets.QGraphicsGridLayout()
         self.layout.setContentsMargins(1,1,1,1)
         self.setLayout(self.layout)
         self.layout.setHorizontalSpacing(0)
@@ -181,10 +180,13 @@ class PlotItem(GraphicsWidget):
         self.dataItems = []
         self.paramList = {}
         self.avgCurves = {}
-        
+        # Change these properties to adjust the appearance of the averged curve:
+        self.avgPen = fn.mkPen([0, 200, 0])
+        self.avgShadowPen = fn.mkPen([0, 0, 0], width=4) # the previous default of [0,0,0,100] prevent fast drawing of the wide shadow line
+
         ### Set up context menu
         
-        w = QtGui.QWidget()
+        w = QtWidgets.QWidget()
         self.ctrl = c = ui_template.Ui_Form()
         c.setupUi(w)
         dv = QtGui.QDoubleValidator(self)
@@ -199,13 +201,13 @@ class PlotItem(GraphicsWidget):
         ]
         
         
-        self.ctrlMenu = QtGui.QMenu()
+        self.ctrlMenu = QtWidgets.QMenu()
         
         self.ctrlMenu.setTitle(translate("PlotItem", 'Plot Options'))
         self.subMenus = []
         for name, grp in menuItems:
-            sm = QtGui.QMenu(name)
-            act = QtGui.QWidgetAction(self)
+            sm = QtWidgets.QMenu(name)
+            act = QtWidgets.QWidgetAction(self)
             act.setDefaultWidget(grp)
             sm.addAction(act)
             self.subMenus.append(sm)
@@ -241,7 +243,7 @@ class PlotItem(GraphicsWidget):
         self.ctrl.avgParamList.itemClicked.connect(self.avgParamListClicked)
         self.ctrl.averageGroup.toggled.connect(self.avgToggled)
         
-        self.ctrl.maxTracesCheck.toggled.connect(self.updateDecimation)
+        self.ctrl.maxTracesCheck.toggled.connect(self._handle_max_traces_toggle)
         self.ctrl.forgetTracesCheck.toggled.connect(self.updateDecimation)
         self.ctrl.maxTracesSpin.valueChanged.connect(self.updateDecimation)
         
@@ -252,7 +254,7 @@ class PlotItem(GraphicsWidget):
                 labels[label] = kargs[label]
                 del kargs[label]
         for k in labels:
-            if isinstance(labels[k], basestring):
+            if isinstance(labels[k], str):
                 labels[k] = (labels[k],)
             self.setLabel(k, *labels[k])
                 
@@ -343,7 +345,7 @@ class PlotItem(GraphicsWidget):
         
     def setLogMode(self, x=None, y=None):
         """
-        Set log scaling for x and/or y axes.
+        Set log scaling for `x` and/or `y` axes.
         This informs PlotDataItems to transform logarithmically and switches
         the axes to use log ticking. 
         
@@ -476,8 +478,8 @@ class PlotItem(GraphicsWidget):
         ### Create a new curve if needed
         if key not in self.avgCurves:
             plot = PlotDataItem()
-            plot.setPen(fn.mkPen([0, 200, 0]))
-            plot.setShadowPen(fn.mkPen([0, 0, 0, 100], width=3))
+            plot.setPen( self.avgPen )
+            plot.setShadowPen(  self.avgShadowPen )
             plot.setAlpha(1.0, False)
             plot.setZValue(100)
             self.addItem(plot, skipAverage=True)
@@ -519,8 +521,9 @@ class PlotItem(GraphicsWidget):
     def addItem(self, item, *args, **kargs):
         """
         Add a graphics item to the view box. 
-        If the item has plot data (PlotDataItem, PlotCurveItem, ScatterPlotItem), it may
-        be included in analysis performed by the PlotItem.
+        If the item has plot data (:class:`~pyqtgrpah.PlotDataItem`, 
+        :class:`~pyqtgraph.PlotCurveItem`, :class:`~pyqtgraph.ScatterPlotItem`), 
+        it may be included in analysis performed by the PlotItem.
         """
         if item in self.items:
             warnings.warn('Item already added to PlotItem, ignoring.')
@@ -578,7 +581,8 @@ class PlotItem(GraphicsWidget):
         self.addItem(item, *args)
         
     def listDataItems(self):
-        """Return a list of all data items (PlotDataItem, PlotCurveItem, ScatterPlotItem, etc)
+        """Return a list of all data items (:class:`~pyqtgrpah.PlotDataItem`, 
+        :class:`~pyqtgraph.PlotCurveItem`, :class:`~pyqtgraph.ScatterPlotItem`, etc)
         contained in this PlotItem."""
         return self.dataItems[:]
         
@@ -587,16 +591,15 @@ class PlotItem(GraphicsWidget):
             'PlotItem.addCurve is deprecated and will be removed in 0.13. '
             'Use PlotItem.addItem instead.',
             DeprecationWarning, stacklevel=2
-        )    
-
+        )
         self.addItem(c, params)
 
     def addLine(self, x=None, y=None, z=None, **kwds):
         """
-        Create an InfiniteLine and add to the plot. 
+        Create an :class:`~pyqtgraph.InfiniteLine` and add to the plot. 
         
-        If *x* is specified,
-        the line will be vertical. If *y* is specified, the line will be
+        If `x` is specified,
+        the line will be vertical. If `y` is specified, the line will be
         horizontal. All extra keyword arguments are passed to
         :func:`InfiniteLine.__init__() <pyqtgraph.InfiniteLine.__init__>`.
         Returns the item created.
@@ -607,11 +610,11 @@ class PlotItem(GraphicsWidget):
         self.addItem(line)
         if z is not None:
             line.setZValue(z)
-        return line        
+        return line
 
     def removeItem(self, item):
         """
-        Remove an item from the internal ViewBox.
+        Remove an item from the PlotItem's :class:`~pyqtgraph.ViewBox`.
         """
         if not item in self.items:
             return
@@ -631,7 +634,7 @@ class PlotItem(GraphicsWidget):
 
     def clear(self):
         """
-        Remove all items from the ViewBox.
+        Remove all items from the PlotItem's :class:`~pyqtgraph.ViewBox`.
         """
         for i in self.items[:]:
             self.removeItem(i)
@@ -643,13 +646,17 @@ class PlotItem(GraphicsWidget):
         self.avgCurves = {}        
     
     def plot(self, *args, **kargs):
+#        **Additional arguments:**
         """
         Add and return a new plot.
         See :func:`PlotDataItem.__init__ <pyqtgraph.PlotDataItem.__init__>` for data arguments
         
-        Extra allowed arguments are:
-            clear    - clear all plots before displaying new data
-            params   - meta-parameters to associate with this data
+        **Additional allowed arguments**
+        
+        ========= =================================================================
+        `clear`   clears all plots before displaying new data
+        `params`  sets meta-parameters to associate with this data
+        ========= =================================================================
         """
         clear = kargs.get('clear', False)
         params = kargs.get('params', None)
@@ -667,14 +674,14 @@ class PlotItem(GraphicsWidget):
 
     def addLegend(self, offset=(30, 30), **kwargs):
         """
-        Create a new :class:`~pyqtgraph.LegendItem` and anchor it over the
-        internal ViewBox. Plots will be automatically displayed in the legend
-        if they are created with the 'name' argument.
+        Create a new :class:`~pyqtgraph.LegendItem` and anchor it over the internal 
+        :class:`~pyqtgraph.ViewBox`. Plots added after this will be automatically 
+        displayed in the legend if they are created with a 'name' argument.
 
-        If a LegendItem has already been created using this method, that
-        item will be returned rather than creating a new one.
+        If a :class:`~pyqtGraph.LegendItem` has already been created using this method, 
+        that item will be returned rather than creating a new one.
 
-        Accepts the same arguments as :meth:`~pyqtgraph.LegendItem`.
+        Accepts the same arguments as :func:`~pyqtgraph.LegendItem.__init__`.
         """
 
         if self.legend is None:
@@ -682,6 +689,19 @@ class PlotItem(GraphicsWidget):
             self.legend.setParentItem(self.vb)
         return self.legend
         
+    def addColorBar(self, image, **kargs):
+        """
+        Adds a color bar linked to the ImageItem specified by `image`.
+        AAdditional parameters will be passed to the `pyqtgraph.ColorBarItem`.
+        
+        A call like `plot.addColorBar(img, colorMap='viridis')` is a convenient
+        method to assign and show a color map.
+        """
+        from ..ColorBarItem import ColorBarItem # avoid circular import
+        bar = ColorBarItem(**kargs)
+        bar.setImageItem( image, insert_in=self )
+        return bar
+
     def scatterPlot(self, *args, **kargs):
         if 'pen' in kargs:
             kargs['symbolPen'] = kargs['pen']
@@ -711,7 +731,7 @@ class PlotItem(GraphicsWidget):
                 ## If the parameter is not in the list, add it.
                 matches = self.ctrl.avgParamList.findItems(p, QtCore.Qt.MatchFlag.MatchExactly)
                 if len(matches) == 0:
-                    i = QtGui.QListWidgetItem(p)
+                    i = QtWidgets.QListWidgetItem(p)
                     if p in self.paramList and self.paramList[p] is True:
                         i.setCheckState(QtCore.Qt.CheckState.Checked)
                     else:
@@ -764,9 +784,8 @@ class PlotItem(GraphicsWidget):
 
             for item in self.curves:
                 if isinstance(item, PlotCurveItem):
-                    color = fn.colorStr(item.pen.color())
-                    opacity = item.pen.color().alpha() / 255.
-                    color = color[:6]
+                    color = item.pen.color()
+                    hrrggbb, opacity = color.name(), color.alphaF()
                     x, y = item.getData()
                     mask = (x > xRange[0]) * (x < xRange[1])
                     mask[:-1] += mask[1:]
@@ -781,9 +800,9 @@ class PlotItem(GraphicsWidget):
                     # fh.write('<g fill="none" stroke="#%s" '
                     #          'stroke-opacity="1" stroke-width="1">\n' % (
                     #           color, ))
-                    fh.write('<path fill="none" stroke="#%s" '
+                    fh.write('<path fill="none" stroke="%s" '
                              'stroke-opacity="%f" stroke-width="1" '
-                             'd="M%f,%f ' % (color, opacity, x[0], y[0]))
+                             'd="M%f,%f ' % (hrrggbb, opacity, x[0], y[0]))
                     for i in range(1, len(x)):
                         fh.write('L%f,%f ' % (x[i], y[i]))
 
@@ -799,15 +818,14 @@ class PlotItem(GraphicsWidget):
                         pos = point.pos()
                         if not rect.contains(pos):
                             continue
-                        color = fn.colorStr(point.brush.color())
-                        opacity = point.brush.color().alpha() / 255.
-                        color = color[:6]
+                        color = point.brush.color()
+                        hrrggbb, opacity = color.name(), color.alphaF()
                         x = pos.x() * sx
                         y = pos.y() * sy
 
-                        fh.write('<circle cx="%f" cy="%f" r="1" fill="#%s" '
+                        fh.write('<circle cx="%f" cy="%f" r="1" fill="%s" '
                                  'stroke="none" fill-opacity="%f"/>\n' % (
-                                    x, y, color, opacity))
+                                    x, y, hrrggbb, opacity))
 
             fh.write("</svg>\n")
 
@@ -908,10 +926,10 @@ class PlotItem(GraphicsWidget):
         for i in self.items:
             if hasattr(i, 'setLogMode'):
                 i.setLogMode(x,y)
-        self.getAxis('bottom').setLogMode(x)
-        self.getAxis('top').setLogMode(x)
-        self.getAxis('left').setLogMode(y)
-        self.getAxis('right').setLogMode(y)
+        self.getAxis('bottom').setLogMode(x, y)
+        self.getAxis('top').setLogMode(x, y)
+        self.getAxis('left').setLogMode(x, y)
+        self.getAxis('right').setLogMode(x, y)
         self.enableAutoRange()
         self.recomputeAverages()
     
@@ -933,20 +951,26 @@ class PlotItem(GraphicsWidget):
         
         
     def setDownsampling(self, ds=None, auto=None, mode=None):
-        """Change the default downsampling mode for all PlotDataItems managed by this plot.
+        """
+        Changes the default downsampling mode for all :class:`~pyqtgraph.PlotDataItem` managed by this plot.
         
-        =============== =================================================================
+        =============== ====================================================================
         **Arguments:**
         ds              (int) Reduce visible plot samples by this factor, or
+
                         (bool) To enable/disable downsampling without changing the value.
-        auto            (bool) If True, automatically pick *ds* based on visible range
-        mode            'subsample': Downsample by taking the first of N samples.
-                        This method is fastest and least accurate.
+
+        auto            (bool) If `True`, automatically pick ``ds`` based on visible range
+
+        mode            'subsample': Downsample by taking the first of N samples. This 
+                        method is fastest but least accurate.
+
                         'mean': Downsample by taking the mean of N samples.
-                        'peak': Downsample by drawing a saw wave that follows the min
-                        and max of the original data. This method produces the best
-                        visual representation of the data but is slower.
-        =============== =================================================================
+
+                        'peak': Downsample by drawing a saw wave that follows the min and 
+                        max of the original data. This method produces the best visual 
+                        representation of the data but is slower.
+        =============== ====================================================================
         """
         if ds is not None:
             if ds is False:
@@ -994,21 +1018,41 @@ class PlotItem(GraphicsWidget):
             method = 'mean'
         elif self.ctrl.peakRadio.isChecked():
             method = 'peak'
+        else:
+            raise ValueError("one of the method radios must be selected for: 'subsample', 'mean', or 'peak'.")
         
         return ds, auto, method
         
     def setClipToView(self, clip):
-        """Set the default clip-to-view mode for all PlotDataItems managed by this plot.
-        If *clip* is True, then PlotDataItems will attempt to draw only points within the visible
+        """Set the default clip-to-view mode for all :class:`~pyqtgraph.PlotDataItem`s managed by this plot.
+        If *clip* is `True`, then PlotDataItems will attempt to draw only points within the visible
         range of the ViewBox."""
         self.ctrl.clipToViewCheck.setChecked(clip)
         
     def clipToViewMode(self):
         return self.ctrl.clipToViewCheck.isChecked()
-
+    
+    def _handle_max_traces_toggle(self, check_state):
+        if check_state:
+            self.updateDecimation()
+        else:
+            for curve in self.curves:
+                curve.show()
+    
     def updateDecimation(self):
+        """
+        Reduce or increase number of visible curves according to value set by the `Max Traces` spinner,
+        if `Max Traces` is checked in the context menu. Destroy curves that are not visible if 
+        `forget traces` is checked. In most cases, this function is called automaticaly when the 
+        `Max Traces` GUI elements are triggered. It is also alled when the state of PlotItem is updated,
+        its state is restored, or new items added added/removed.
+        
+        This can cause an unexpected or conflicting state of curve visibility (or destruction) if curve
+        visibilities are controlled externally. In the case of external control it is advised to disable
+        the `Max Traces` checkbox (or context menu) to prevent unexpected curve state changes.
+        """
         if not self.ctrl.maxTracesCheck.isChecked():
-            numCurves = len(self.curves)
+            return
         else:
             numCurves = self.ctrl.maxTracesSpin.value()
 
@@ -1070,7 +1114,7 @@ class PlotItem(GraphicsWidget):
         """
         Enable or disable the context menu for this PlotItem.
         By default, the ViewBox's context menu will also be affected.
-        (use enableViewBoxMenu=None to leave the ViewBox unchanged)
+        (use ``enableViewBoxMenu=None`` to leave the ViewBox unchanged)
         """
         self._menuEnabled = enableMenu
         if enableViewBoxMenu is None:
@@ -1108,7 +1152,7 @@ class PlotItem(GraphicsWidget):
         
     def setLabel(self, axis, text=None, units=None, unitPrefix=None, **args):
         """
-        Set the label for an axis. Basic HTML formatting is allowed.
+        Sets the label for an axis. Basic HTML formatting is allowed.
         
         ==============  =================================================================
         **Arguments:**
@@ -1127,13 +1171,13 @@ class PlotItem(GraphicsWidget):
         """
         Convenience function allowing multiple labels and/or title to be set in one call.
         Keyword arguments can be 'title', 'left', 'bottom', 'right', or 'top'.
-        Values may be strings or a tuple of arguments to pass to setLabel.
+        Values may be strings or a tuple of arguments to pass to :func:`setLabel`.
         """
         for k,v in kwds.items():
             if k == 'title':
                 self.setTitle(v)
             else:
-                if isinstance(v, basestring):
+                if isinstance(v, str):
                     v = (v,)
                 self.setLabel(k, *v)
         
@@ -1304,7 +1348,7 @@ class PlotItem(GraphicsWidget):
         self.fileDialog = FileDialog()
         if PlotItem.lastFileDir is not None:
             self.fileDialog.setDirectory(PlotItem.lastFileDir)
-        self.fileDialog.setFileMode(QtGui.QFileDialog.FileMode.AnyFile)
-        self.fileDialog.setAcceptMode(QtGui.QFileDialog.AcceptMode.AcceptSave)
+        self.fileDialog.setFileMode(QtWidgets.QFileDialog.FileMode.AnyFile)
+        self.fileDialog.setAcceptMode(QtWidgets.QFileDialog.AcceptMode.AcceptSave)
         self.fileDialog.show()
         self.fileDialog.fileSelected.connect(handler)
