@@ -1,11 +1,11 @@
 if __name__ == '__main__':
-    import os, sys
+    import os
+    import sys
     path = os.path.abspath(os.path.dirname(__file__))
     sys.path.insert(0, os.path.join(path, '..', '..'))
 
-from ..Qt import QtGui, QtCore
 from .. import functions as fn
-import weakref
+from ..Qt import QtGui, QtWidgets
 from .UIGraphicsItem import UIGraphicsItem
 
 __all__ = ['VTickGroup']
@@ -39,7 +39,7 @@ class VTickGroup(UIGraphicsItem):
         if pen is None:
             pen = (200, 200, 200)
             
-        self.path = QtGui.QGraphicsPathItem()
+        self.path = QtWidgets.QGraphicsPathItem()
         
         self.ticks = []
         self.xvals = []
@@ -90,10 +90,10 @@ class VTickGroup(UIGraphicsItem):
         br = self.boundingRect()
         h = br.height()
         br.setY(br.y() + self.yrange[0] * h)
-        br.setHeight(h - (1.0-self.yrange[1]) * h)
+        br.setHeight((self.yrange[1] - self.yrange[0]) * h)
         p.translate(0, br.y())
         p.scale(1.0, br.height())
         p.setPen(self.pen)
         p.drawPath(self.path)
 
-    
\ No newline at end of file
+    
