@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 DEPRECATED:  The classes below are convenience classes that create a new window
 containting a single, specific widget. These classes are now unnecessary because
@@ -8,13 +7,13 @@ show() method.
 
 __all__ = ['GraphicsWindow', 'TabWindow', 'PlotWindow', 'ImageWindow']
 
-from .Qt import QtCore, QtGui, mkQApp
-from .widgets.PlotWidget import *
-from .imageview import *
-from .widgets.GraphicsLayoutWidget import GraphicsLayoutWidget
-from .widgets.GraphicsView import GraphicsView
 import warnings
 
+from .imageview import *
+from .Qt import QtCore, QtWidgets, mkQApp
+from .widgets.GraphicsLayoutWidget import GraphicsLayoutWidget
+from .widgets.PlotWidget import *
+
 
 class GraphicsWindow(GraphicsLayoutWidget):
     """
@@ -37,7 +36,7 @@ class GraphicsWindow(GraphicsLayoutWidget):
         self.show()
         
 
-class TabWindow(QtGui.QMainWindow):
+class TabWindow(QtWidgets.QMainWindow):
     """
     (deprecated)
     """
@@ -47,9 +46,9 @@ class TabWindow(QtGui.QMainWindow):
             DeprecationWarning, stacklevel=2
         )
         mkQApp()
-        QtGui.QMainWindow.__init__(self)
+        QtWidgets.QMainWindow.__init__(self)
         self.resize(*size)
-        self.cw = QtGui.QTabWidget()
+        self.cw = QtWidgets.QTabWidget()
         self.setCentralWidget(self.cw)
         if title is not None:
             self.setWindowTitle(title)
@@ -72,7 +71,7 @@ class PlotWindow(PlotWidget):
             DeprecationWarning, stacklevel=2
         )    
         mkQApp()
-        self.win = QtGui.QMainWindow()
+        self.win = QtWidgets.QMainWindow()
         PlotWidget.__init__(self, **kargs)
         self.win.setCentralWidget(self)
         for m in ['resize']:
@@ -99,7 +98,7 @@ class ImageWindow(ImageView):
             DeprecationWarning, stacklevel=2
         ) 
         mkQApp()
-        self.win = QtGui.QMainWindow()
+        self.win = QtWidgets.QMainWindow()
         self.win.resize(800,600)
         if 'title' in kargs:
             self.win.setWindowTitle(kargs['title'])
