@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 ImageView.py -  Widget for basic image dispay and analysis
 Copyright 2010  Luke Campagnola
@@ -12,30 +11,34 @@ Widget used for displaying 2D or 3D data. Features:
   - ROI plotting
   - Image normalization through a variety of methods
 """
-import os, sys
+import importlib
+import os
+from math import log10
+from time import perf_counter
+
 import numpy as np
 
-from ..Qt import QtCore, QtGui, QT_LIB
-import importlib
+from .. import functions as fn
+from ..Qt import QT_LIB, QtCore, QtGui, QtWidgets
+
 ui_template = importlib.import_module(
     f'.ImageViewTemplate_{QT_LIB.lower()}', package=__package__)
-    
+
+from .. import debug as debug
+from .. import getConfigOption
+from ..graphicsItems.GradientEditorItem import addGradientListToDocstring
 from ..graphicsItems.ImageItem import *
-from ..graphicsItems.ROI import *
-from ..graphicsItems.LinearRegionItem import *
 from ..graphicsItems.InfiniteLine import *
+from ..graphicsItems.LinearRegionItem import *
+from ..graphicsItems.ROI import *
 from ..graphicsItems.ViewBox import *
 from ..graphicsItems.VTickGroup import VTickGroup
-from ..graphicsItems.GradientEditorItem import addGradientListToDocstring
-from .. import ptime as ptime
-from .. import debug as debug
 from ..SignalProxy import SignalProxy
-from .. import getConfigOption
 
 try:
-    from bottleneck import nanmin, nanmax
+    from bottleneck import nanmax, nanmin
 except ImportError:
-    from numpy import nanmin, nanmax
+    from numpy import nanmax, nanmin
 
 translate = QtCore.QCoreApplication.translate
 
@@ -46,20 +49,20 @@ class PlotROI(ROI):
         self.addRotateHandle([0, 0], [0.5, 0.5])
 
 
-class ImageView(QtGui.QWidget):
+class ImageView(QtWidgets.QWidget):
     """
     Widget used for display and analysis of image data.
     Implements many features:
     
-    * Displays 2D and 3D image data. For 3D data, a z-axis
-      slider is displayed allowing the user to select which frame is displayed.
-    * Displays histogram of image data with movable region defining the dark/light levels
-    * Editable gradient provides a color lookup table 
-    * Frame slider may also be moved using left/right arrow keys as well as pgup, pgdn, home, and end.
-    * Basic analysis features including:
-    
-        * ROI and embedded plot for measuring image values across frames
-        * Image normalization / background subtraction 
+      * Displays 2D and 3D image data. For 3D data, a z-axis
+        slider is displayed allowing the user to select which frame is displayed.
+      * Displays histogram of image data with movable region defining the dark/light levels
+      * Editable gradient provides a color lookup table
+      * Frame slider may also be moved using left/right arrow keys as well as pgup, pgdn, home, and end.
+      * Basic analysis features including:
+
+          * ROI and embedded plot for measuring image values across frames
+          * Image normalization / background subtraction
     
     Basic Usage::
     
@@ -69,13 +72,13 @@ class ImageView(QtGui.QWidget):
         
     **Keyboard interaction**
     
-    * left/right arrows step forward/backward 1 frame when pressed,
-      seek at 20fps when held.
-    * up/down arrows seek at 100fps
-    * pgup/pgdn seek at 1000fps
-    * home/end seek immediately to the first/last frame
-    * space begins playing frames. If time values (in seconds) are given 
-      for each frame, then playback is in realtime.
+      * left/right arrows step forward/backward 1 frame when pressed,
+        seek at 20fps when held.
+      * up/down arrows seek at 100fps
+      * pgup/pgdn seek at 1000fps
+      * home/end seek immediately to the first/last frame
+      * space begins playing frames. If time values (in seconds) are given
+        for each frame, then playback is in realtime.
     """
     sigTimeChanged = QtCore.Signal(object, object)
     sigProcessingChanged = QtCore.Signal(object)
@@ -113,7 +116,7 @@ class ImageView(QtGui.QWidget):
                 
             pg.ImageView(view=pg.PlotItem())
         """
-        QtGui.QWidget.__init__(self, parent, *args)
+        QtWidgets.QWidget.__init__(self, parent, *args)
         self._imageLevels = None  # [(min, max), ...] per channel image metrics
         self.levelMin = None    # min / max levels across all channels
         self.levelMax = None
@@ -125,7 +128,6 @@ class ImageView(QtGui.QWidget):
         self.ui = ui_template.Ui_Form()
         self.ui.setupUi(self)
         self.scene = self.ui.graphicsView.scene()
-        self.ui.histogram.setLevelMode(levelMode)
         
         self.ignorePlaying = False
         
@@ -137,15 +139,6 @@ class ImageView(QtGui.QWidget):
         self.view.setAspectLocked(True)
         self.view.invertY()
         
-        if imageItem is None:
-            self.imageItem = ImageItem()
-        else:
-            self.imageItem = imageItem
-        self.view.addItem(self.imageItem)
-        self.currentIndex = 0
-        
-        self.ui.histogram.setImageItem(self.imageItem)
-        
         self.menu = None
         
         self.ui.normGroup.hide()
@@ -160,7 +153,6 @@ class ImageView(QtGui.QWidget):
         self.view.addItem(self.normRoi)
         self.normRoi.hide()
         self.roiCurves = []
-        self.roiCurve = self.ui.roiPlot.plot()
         self.timeLine = InfiniteLine(0, movable=True)
         if getConfigOption('background')=='w':
             self.timeLine.setPen((20, 80,80, 200))
@@ -169,6 +161,18 @@ class ImageView(QtGui.QWidget):
         self.timeLine.setZValue(1)
         self.ui.roiPlot.addItem(self.timeLine)
         self.ui.splitter.setSizes([self.height()-35, 35])
+
+        # init imageItem and histogram
+        if imageItem is None:
+            self.imageItem = ImageItem()
+        else:
+            self.imageItem = imageItem
+            self.setImage(imageItem.image, autoRange=False, autoLevels=False, transform=imageItem.transform())
+        self.view.addItem(self.imageItem)
+        self.currentIndex = 0
+        
+        self.ui.histogram.setImageItem(self.imageItem)
+        self.ui.histogram.setLevelMode(levelMode)
         
         # make splitter an unchangeable small grey line:
         s = self.ui.splitter
@@ -218,7 +222,7 @@ class ImageView(QtGui.QWidget):
         self.ui.roiPlot.registerPlot(self.name + '_ROI')
         self.view.register(self.name)
         
-        self.noRepeatKeys = [QtCore.Qt.Key_Right, QtCore.Qt.Key_Left, QtCore.Qt.Key_Up, QtCore.Qt.Key_Down, QtCore.Qt.Key_PageUp, QtCore.Qt.Key_PageDown]
+        self.noRepeatKeys = [QtCore.Qt.Key.Key_Right, QtCore.Qt.Key.Key_Left, QtCore.Qt.Key.Key_Up, QtCore.Qt.Key.Key_Down, QtCore.Qt.Key.Key_PageUp, QtCore.Qt.Key.Key_PageDown]
         
         self.roiClicked() ## initialize roi plot to correct shape / visibility
 
@@ -272,7 +276,7 @@ class ImageView(QtGui.QWidget):
         
         if not isinstance(img, np.ndarray):
             required = ['dtype', 'max', 'min', 'ndim', 'shape', 'size']
-            if not all([hasattr(img, attr) for attr in required]):
+            if not all(hasattr(img, attr) for attr in required):
                 raise TypeError("Image must be NumPy array or any object "
                                 "that provides compatible attributes/methods:\n"
                                 "  %s" % str(required))
@@ -356,13 +360,15 @@ class ImageView(QtGui.QWidget):
         
         profiler()
 
-        self.imageItem.resetTransform()
-        if scale is not None:
-            self.imageItem.scale(*scale)
-        if pos is not None:
-            self.imageItem.setPos(*pos)
-        if transform is not None:
-            self.imageItem.setTransform(transform)
+        if transform is None:
+            transform = QtGui.QTransform()
+            # note that the order of transform is
+            #   scale followed by translate
+            if pos is not None:
+                transform.translate(*pos)
+            if scale is not None:
+                transform.scale(*scale)
+        self.imageItem.setTransform(transform)
 
         profiler()
 
@@ -388,7 +394,7 @@ class ImageView(QtGui.QWidget):
             self.playTimer.stop()
             return
             
-        self.lastPlayTime = ptime.time()
+        self.lastPlayTime = perf_counter()
         if not self.playTimer.isActive():
             self.playTimer.start(16)
             
@@ -433,17 +439,17 @@ class ImageView(QtGui.QWidget):
             super().keyPressEvent(ev)
             return
 
-        if ev.key() == QtCore.Qt.Key_Space:
+        if ev.key() == QtCore.Qt.Key.Key_Space:
             if self.playRate == 0:
                 self.play()
             else:
                 self.play(0)
             ev.accept()
-        elif ev.key() == QtCore.Qt.Key_Home:
+        elif ev.key() == QtCore.Qt.Key.Key_Home:
             self.setCurrentIndex(0)
             self.play(0)
             ev.accept()
-        elif ev.key() == QtCore.Qt.Key_End:
+        elif ev.key() == QtCore.Qt.Key.Key_End:
             self.setCurrentIndex(self.getProcessedImage().shape[0]-1)
             self.play(0)
             ev.accept()
@@ -461,7 +467,7 @@ class ImageView(QtGui.QWidget):
             super().keyReleaseEvent(ev)
             return
 
-        if ev.key() in [QtCore.Qt.Key_Space, QtCore.Qt.Key_Home, QtCore.Qt.Key_End]:
+        if ev.key() in [QtCore.Qt.Key.Key_Space, QtCore.Qt.Key.Key_Home, QtCore.Qt.Key.Key_End]:
             ev.accept()
         elif ev.key() in self.noRepeatKeys:
             ev.accept()
@@ -478,28 +484,28 @@ class ImageView(QtGui.QWidget):
     def evalKeyState(self):
         if len(self.keysPressed) == 1:
             key = list(self.keysPressed.keys())[0]
-            if key == QtCore.Qt.Key_Right:
+            if key == QtCore.Qt.Key.Key_Right:
                 self.play(20)
                 self.jumpFrames(1)
-                self.lastPlayTime = ptime.time() + 0.2  ## 2ms wait before start
-                                                        ## This happens *after* jumpFrames, since it might take longer than 2ms
-            elif key == QtCore.Qt.Key_Left:
+                # effectively pause playback for 0.2 s
+                self.lastPlayTime = perf_counter() + 0.2  
+            elif key == QtCore.Qt.Key.Key_Left:
                 self.play(-20)
                 self.jumpFrames(-1)
-                self.lastPlayTime = ptime.time() + 0.2
-            elif key == QtCore.Qt.Key_Up:
+                self.lastPlayTime = perf_counter() + 0.2
+            elif key == QtCore.Qt.Key.Key_Up:
                 self.play(-100)
-            elif key == QtCore.Qt.Key_Down:
+            elif key == QtCore.Qt.Key.Key_Down:
                 self.play(100)
-            elif key == QtCore.Qt.Key_PageUp:
+            elif key == QtCore.Qt.Key.Key_PageUp:
                 self.play(-1000)
-            elif key == QtCore.Qt.Key_PageDown:
+            elif key == QtCore.Qt.Key.Key_PageDown:
                 self.play(1000)
         else:
             self.play(0)
         
     def timeout(self):
-        now = ptime.time()
+        now = perf_counter()
         dt = now - self.lastPlayTime
         if dt < 0:
             return
@@ -512,7 +518,7 @@ class ImageView(QtGui.QWidget):
         
     def setCurrentIndex(self, ind):
         """Set the currently displayed frame index."""
-        index = np.clip(ind, 0, self.getProcessedImage().shape[self.axes['t']]-1)
+        index = fn.clip_scalar(ind, 0, self.getProcessedImage().shape[self.axes['t']]-1)
         self.ignorePlaying = True
         # Implicitly call timeLineChanged
         self.timeLine.setValue(self.tVals[index])
@@ -561,12 +567,12 @@ class ImageView(QtGui.QWidget):
         if self.ui.roiBtn.isChecked():
             showRoiPlot = True
             self.roi.show()
-            #self.ui.roiPlot.show()
             self.ui.roiPlot.setMouseEnabled(True, True)
             self.ui.splitter.setSizes([int(self.height()*0.6), int(self.height()*0.4)])
             self.ui.splitter.handle(1).setEnabled(True)
-            self.roiCurve.show()
             self.roiChanged()
+            for c in self.roiCurves:
+                c.show()
             self.ui.roiPlot.showAxis('left')
         else:
             self.roi.hide()
@@ -582,13 +588,11 @@ class ImageView(QtGui.QWidget):
             self.ui.roiPlot.setXRange(mn, mx, padding=0.01)
             self.timeLine.show()
             self.timeLine.setBounds([mn, mx])
-            self.ui.roiPlot.show()
             if not self.ui.roiBtn.isChecked():
                 self.ui.splitter.setSizes([self.height()-35, 35])
                 self.ui.splitter.handle(1).setEnabled(False)
         else:
             self.timeLine.hide()
-            #self.ui.roiPlot.hide()
             
         self.ui.roiPlot.setVisible(showRoiPlot)
 
@@ -807,7 +811,7 @@ class ImageView(QtGui.QWidget):
         img = self.getProcessedImage()
         if self.hasTimeAxis():
             base, ext = os.path.splitext(fileName)
-            fmt = "%%s%%0%dd%%s" % int(np.log10(img.shape[0])+1)
+            fmt = "%%s%%0%dd%%s" % int(log10(img.shape[0])+1)
             for i in range(img.shape[0]):
                 self.imageItem.setImage(img[i], autoLevels=False)
                 self.imageItem.save(fmt % (base, i, ext))
@@ -816,7 +820,7 @@ class ImageView(QtGui.QWidget):
             self.imageItem.save(fileName)
             
     def exportClicked(self):
-        fileName = QtGui.QFileDialog.getSaveFileName()
+        fileName = QtWidgets.QFileDialog.getSaveFileName()
         if isinstance(fileName, tuple):
             fileName = fileName[0]  # Qt4/5 API difference
         if fileName == '':
@@ -824,7 +828,7 @@ class ImageView(QtGui.QWidget):
         self.export(str(fileName))
         
     def buildMenu(self):
-        self.menu = QtGui.QMenu()
+        self.menu = QtWidgets.QMenu()
         self.normAction = QtGui.QAction(translate("ImageView", "Normalization"), self.menu)
         self.normAction.setCheckable(True)
         self.normAction.toggled.connect(self.normToggled)
