@@ -1,8 +1,9 @@
+from OpenGL.GL import *  # noqa
 import numpy as np
 
-from OpenGL.GL import *
-from .. GLGraphicsItem import GLGraphicsItem
 from ... import QtGui
+from ... import functions as fn
+from ..GLGraphicsItem import GLGraphicsItem
 
 __all__ = ['GLGridItem']
 
@@ -10,10 +11,10 @@ class GLGridItem(GLGraphicsItem):
     """
     **Bases:** :class:`GLGraphicsItem <pyqtgraph.opengl.GLGraphicsItem>`
     
-    Displays a wire-grame grid. 
+    Displays a wire-frame grid. 
     """
     
-    def __init__(self, size=None, color=None, antialias=True, glOptions='translucent'):
+    def __init__(self, size=None, color=(255, 255, 255, 76.5), antialias=True, glOptions='translucent'):
         GLGraphicsItem.__init__(self)
         self.setGLOptions(glOptions)
         self.antialias = antialias
@@ -21,6 +22,7 @@ class GLGridItem(GLGraphicsItem):
             size = QtGui.QVector3D(20,20,1)
         self.setSize(size=size)
         self.setSpacing(1, 1, 1)
+        self.setColor(color)
     
     def setSize(self, x=None, y=None, z=None, size=None):
         """
@@ -52,6 +54,14 @@ class GLGridItem(GLGraphicsItem):
     def spacing(self):
         return self.__spacing[:]
         
+    def setColor(self, color):
+        """Set the color of the grid. Arguments are the same as those accepted by functions.mkColor()"""
+        self.__color = fn.mkColor(color)
+        self.update()
+
+    def color(self):
+        return self.__color
+
     def paint(self):
         self.setupGLState()
         
@@ -66,8 +76,8 @@ class GLGridItem(GLGraphicsItem):
         x,y,z = self.size()
         xs,ys,zs = self.spacing()
         xvals = np.arange(-x/2., x/2. + xs*0.001, xs) 
-        yvals = np.arange(-y/2., y/2. + ys*0.001, ys) 
-        glColor4f(1, 1, 1, .3)
+        yvals = np.arange(-y/2., y/2. + ys*0.001, ys)
+        glColor4f(*self.color().getRgbF())
         for x in xvals:
             glVertex3f(x, yvals[0], 0)
             glVertex3f(x,  yvals[-1], 0)
