@@ -1,12 +1,10 @@
-from OpenGL.GL import *
-from .. GLGraphicsItem import GLGraphicsItem
-from .. MeshData import MeshData
-from ...Qt import QtGui
-from .. import shaders
-from ... import functions as fn
+from OpenGL.GL import *  # noqa
 import numpy as np
 
-
+from ...Qt import QtGui
+from .. import shaders
+from ..GLGraphicsItem import GLGraphicsItem
+from ..MeshData import MeshData
 
 __all__ = ['GLMeshItem']
 
@@ -181,7 +179,7 @@ class GLMeshItem(GLGraphicsItem):
                     if self.colors is None:
                         color = self.opts['color']
                         if isinstance(color, QtGui.QColor):
-                            glColor4f(*fn.glColor(color))
+                            glColor4f(*color.getRgbF())
                         else:
                             glColor4f(*color)
                     else:
@@ -213,7 +211,7 @@ class GLMeshItem(GLGraphicsItem):
                 if self.edgeColors is None:
                     color = self.opts['edgeColor']
                     if isinstance(color, QtGui.QColor):
-                        glColor4f(*fn.glColor(color))
+                        glColor4f(*color.getRgbF())
                     else:
                         glColor4f(*color)
                 else:
