@@ -1,31 +1,30 @@
 import sys
-from ..Qt import QtGui, QtCore
-from ..SignalProxy import SignalProxy
 from collections import OrderedDict
-from ..python2_3 import asUnicode, basestring
+
+from ..Qt import QtWidgets
 
 __all__ = ['ComboBox']
 
-class ComboBox(QtGui.QComboBox):
+class ComboBox(QtWidgets.QComboBox):
     """Extends QComboBox to add extra functionality.
 
-    * Handles dict mappings -- user selects a text key, and the ComboBox indicates
-      the selected value.
-    * Requires item strings to be unique
-    * Remembers selected value if list is cleared and subsequently repopulated
-    * setItems() replaces the items in the ComboBox and blocks signals if the
-      value ultimately does not change.
+      * Handles dict mappings -- user selects a text key, and the ComboBox indicates
+        the selected value.
+      * Requires item strings to be unique
+      * Remembers selected value if list is cleared and subsequently repopulated
+      * setItems() replaces the items in the ComboBox and blocks signals if the
+        value ultimately does not change.
     """
     
     
     def __init__(self, parent=None, items=None, default=None):
-        QtGui.QComboBox.__init__(self, parent)
+        QtWidgets.QComboBox.__init__(self, parent)
         self.currentIndexChanged.connect(self.indexChanged)
         self._ignoreIndexChange = False
         
         #self.value = default
         if 'darwin' in sys.platform: ## because MacOSX can show names that are wider than the comboBox
-            self.setSizeAdjustPolicy(QtGui.QComboBox.SizeAdjustPolicy.AdjustToContents)
+            self.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToContents)
             #self.setMinimumContentsLength(10)
         self._chosenText = None
         self._items = OrderedDict()
@@ -64,7 +63,7 @@ class ComboBox(QtGui.QComboBox):
         """
         if self.count() == 0:
             return None
-        text = asUnicode(self.currentText())
+        text = self.currentText()
         return self._items[text]
     
     def ignoreIndexChange(func):
@@ -131,10 +130,10 @@ class ComboBox(QtGui.QComboBox):
         # current index has changed; need to remember new 'chosen text'
         if self._ignoreIndexChange:
             return
-        self._chosenText = asUnicode(self.currentText())
+        self._chosenText = self.currentText()
         
     def setCurrentIndex(self, index):
-        QtGui.QComboBox.setCurrentIndex(self, index)
+        QtWidgets.QComboBox.setCurrentIndex(self, index)
         
     def itemsChanged(self):
         # try to set the value to the last one selected, if it is available.
@@ -147,20 +146,20 @@ class ComboBox(QtGui.QComboBox):
     @ignoreIndexChange
     def insertItem(self, *args):
         raise NotImplementedError()
-        #QtGui.QComboBox.insertItem(self, *args)
+        #QtWidgets.QComboBox.insertItem(self, *args)
         #self.itemsChanged()
         
     @ignoreIndexChange
     def insertItems(self, *args):
         raise NotImplementedError()
-        #QtGui.QComboBox.insertItems(self, *args)
+        #QtWidgets.QComboBox.insertItems(self, *args)
         #self.itemsChanged()
     
     @ignoreIndexChange
     def addItem(self, *args, **kwds):
         # Need to handle two different function signatures for QComboBox.addItem
         try:
-            if isinstance(args[0], basestring):
+            if isinstance(args[0], str):
                 text = args[0]
                 if len(args) == 2:
                     value = args[1]
@@ -180,7 +179,7 @@ class ComboBox(QtGui.QComboBox):
             raise Exception('ComboBox already has item named "%s".' % text)
         
         self._items[text] = value
-        QtGui.QComboBox.addItem(self, *args)
+        QtWidgets.QComboBox.addItem(self, *args)
         self.itemsChanged()
         
     def setItemValue(self, name, value):
@@ -207,14 +206,14 @@ class ComboBox(QtGui.QComboBox):
         
         for k,v in items.items():
             self._items[k] = v
-        QtGui.QComboBox.addItems(self, list(texts))
+        QtWidgets.QComboBox.addItems(self, list(texts))
         
         self.itemsChanged()
         
     @ignoreIndexChange
     def clear(self):
         self._items = OrderedDict()
-        QtGui.QComboBox.clear(self)
+        QtWidgets.QComboBox.clear(self)
         self.itemsChanged()
         
     def saveState(self):
@@ -230,7 +229,7 @@ class ComboBox(QtGui.QComboBox):
             except AttributeError:
                 pass
         if data is None:
-            return asUnicode(self.itemText(ind))
+            return self.itemText(ind)
         else:
             return data
         
