@@ -1,13 +1,14 @@
-from ..Qt import QtGui, QtCore
+from math import hypot
 
+from ..Qt import QtCore, QtGui, QtWidgets, mkQApp
 
 __all__ = ['JoystickButton']
 
-class JoystickButton(QtGui.QPushButton):
+class JoystickButton(QtWidgets.QPushButton):
     sigStateChanged = QtCore.Signal(object, object)  ## self, state
     
     def __init__(self, parent=None):
-        QtGui.QPushButton.__init__(self, parent)
+        QtWidgets.QPushButton.__init__(self, parent)
         self.radius = 200
         self.setCheckable(True)
         self.state = None
@@ -18,11 +19,13 @@ class JoystickButton(QtGui.QPushButton):
         
     def mousePressEvent(self, ev):
         self.setChecked(True)
-        self.pressPos = ev.localPos()
+        lpos = ev.position() if hasattr(ev, 'position') else ev.localPos()
+        self.pressPos = lpos
         ev.accept()
         
     def mouseMoveEvent(self, ev):
-        dif = ev.localPos()-self.pressPos
+        lpos = ev.position() if hasattr(ev, 'position') else ev.localPos()
+        dif = lpos - self.pressPos
         self.setState(dif.x(), -dif.y())
         
     def mouseReleaseEvent(self, ev):
@@ -41,7 +44,7 @@ class JoystickButton(QtGui.QPushButton):
         
     def setState(self, *xy):
         xy = list(xy)
-        d = (xy[0]**2 + xy[1]**2)**0.5
+        d = hypot(xy[0], xy[1])  # length
         nxy = [0, 0]
         for i in [0,1]:
             if xy[i] == 0:
@@ -84,8 +87,8 @@ class JoystickButton(QtGui.QPushButton):
         
         
 if __name__ == '__main__':
-    app = pg.mkQApp()
-    w = QtGui.QMainWindow()
+    app = mkQApp()
+    w = QtWidgets.QMainWindow()
     b = JoystickButton()
     w.setCentralWidget(b)
     w.show()
@@ -96,5 +99,4 @@ if __name__ == '__main__':
         
     b.sigStateChanged.connect(fn)
         
-    app.exec_()
-        
\ No newline at end of file
+    app.exec() if hasattr(app, 'exec') else app.exec_()
