@@ -1,31 +1,26 @@
-# -*- coding: utf-8 -*-
-from math import isnan, isinf
-from decimal import Decimal as D  ## Use decimal to avoid accumulating floating-point errors
 import decimal
-import weakref
 import re
+from math import isinf, isnan
 
-from ..Qt import QtGui, QtCore
-from ..python2_3 import asUnicode, basestring
-from ..SignalProxy import SignalProxy
 from .. import functions as fn
-
+from ..Qt import QtCore, QtGui, QtWidgets
+from ..SignalProxy import SignalProxy
 
 __all__ = ['SpinBox']
 
 
-class SpinBox(QtGui.QAbstractSpinBox):
+class SpinBox(QtWidgets.QAbstractSpinBox):
     """
-    **Bases:** QtGui.QAbstractSpinBox
+    **Bases:** QtWidgets.QAbstractSpinBox
     
     Extension of QSpinBox widget for selection of a numerical value.     
     Adds many extra features:
     
-    * SI prefix notation (eg, automatically display "300 mV" instead of "0.003 V")
-    * Float values with linear and decimal stepping (1-9, 10-90, 100-900, etc.)
-    * Option for unbounded values
-    * Delayed signals (allows multiple rapid changes with only one change signal)
-    * Customizable text formatting
+      * SI prefix notation (eg, automatically display "300 mV" instead of "0.003 V")
+      * Float values with linear and decimal stepping (1-9, 10-90, 100-900, etc.)
+      * Option for unbounded values
+      * Delayed signals (allows multiple rapid changes with only one change signal)
+      * Customizable text formatting
     
     =============================  ==============================================
     **Signals:**
@@ -59,14 +54,14 @@ class SpinBox(QtGui.QAbstractSpinBox):
         
         All keyword arguments are passed to :func:`setOpts`.
         """
-        QtGui.QAbstractSpinBox.__init__(self, parent)
+        QtWidgets.QAbstractSpinBox.__init__(self, parent)
         self.lastValEmitted = None
         self.lastText = ''
         self.textValid = True  ## If false, we draw a red border
         self.setMinimumWidth(0)
         self._lastFontHeight = None
         
-        self.setSizePolicy(QtGui.QSizePolicy.Policy.Expanding, QtGui.QSizePolicy.Policy.Preferred)
+        self.setSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Preferred)
         self.errorBox = ErrorBox(self.lineEdit())
         
         self.opts = {
@@ -74,7 +69,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
             'wrapping': False,
            
             ## normal arithmetic step
-            'step': D('0.01'),  ## if 'dec' is false, the spinBox steps by 'step' every time
+            'step': decimal.Decimal('0.01'),  ## if 'dec' is false, the spinBox steps by 'step' every time
                                 ## if 'dec' is True, the step size is relative to the value
                                 ## 'step' needs to be an integral divisor of ten, ie 'step'*n=10 for some integer value of n (but only if dec is True)
             'dec': False,   ## if true, does decimal stepping. ie from 1-10 it steps by 'step', from 10 to 100 it steps by 10*'step', etc. 
@@ -92,16 +87,16 @@ class SpinBox(QtGui.QAbstractSpinBox):
             
             'decimals': 6,
             
-            'format': asUnicode("{scaledValue:.{decimals}g}{suffixGap}{siPrefix}{suffix}"),
+            'format': "{scaledValue:.{decimals}g}{suffixGap}{siPrefix}{suffix}",
             'regex': fn.FLOAT_REGEX,
-            'evalFunc': D,
+            'evalFunc': decimal.Decimal,
             
             'compactHeight': True,  # manually remove extra margin outside of text
         }
         
         self.decOpts = ['step', 'minStep']
         
-        self.val = D(asUnicode(value))  ## Value is precise decimal. Ordinary math not allowed.
+        self.val = decimal.Decimal(str(value))  ## Value is precise decimal. Ordinary math not allowed.
         self.updateText()
         self.skipValidate = False
         self.setCorrectionMode(self.CorrectionMode.CorrectToPreviousValue)
@@ -150,13 +145,13 @@ class SpinBox(QtGui.QAbstractSpinBox):
         format         (str) Formatting string used to generate the text shown. Formatting is
                        done with ``str.format()`` and makes use of several arguments:
                        
-                       * *value* - the unscaled value of the spin box
-                       * *suffix* - the suffix string
-                       * *scaledValue* - the scaled value to use when an SI prefix is present
-                       * *siPrefix* - the SI prefix string (if any), or an empty string if
-                         this feature has been disabled
-                       * *suffixGap* - a single space if a suffix is present, or an empty
-                         string otherwise.
+                         * *value* - the unscaled value of the spin box
+                         * *suffix* - the suffix string
+                         * *scaledValue* - the scaled value to use when an SI prefix is present
+                         * *siPrefix* - the SI prefix string (if any), or an empty string if
+                           this feature has been disabled
+                         * *suffixGap* - a single space if a suffix is present, or an empty
+                           string otherwise.
         regex          (str or RegexObject) Regular expression used to parse the spinbox text.
                        May contain the following group names:
                        
@@ -183,12 +178,12 @@ class SpinBox(QtGui.QAbstractSpinBox):
             elif k == 'max':
                 self.setMaximum(v, update=False)
             elif k in ['step', 'minStep']:
-                self.opts[k] = D(asUnicode(v))
+                self.opts[k] = decimal.Decimal(str(v))
             elif k == 'value':
                 pass   ## don't set value until bounds have been set
             elif k == 'format':
-                self.opts[k] = asUnicode(v)
-            elif k == 'regex' and isinstance(v, basestring):
+                self.opts[k] = str(v)
+            elif k == 'regex' and isinstance(v, str):
                 self.opts[k] = re.compile(v)
             elif k in self.opts:
                 self.opts[k] = v
@@ -222,7 +217,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
                 self.opts['minStep'] = ms
 
             if 'format' not in opts:
-                self.opts['format'] = asUnicode("{value:d}{suffixGap}{suffix}")
+                self.opts['format'] = "{value:d}{suffixGap}{suffix}"
 
         if self.opts['dec']:
             if self.opts.get('minStep') is None:
@@ -236,7 +231,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
     def setMaximum(self, m, update=True):
         """Set the maximum allowed value (or None for no limit)"""
         if m is not None:
-            m = D(asUnicode(m))
+            m = decimal.Decimal(str(m))
         self.opts['bounds'][1] = m
         if update:
             self.setValue()
@@ -244,7 +239,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
     def setMinimum(self, m, update=True):
         """Set the minimum allowed value (or None for no limit)"""
         if m is not None:
-            m = D(asUnicode(m))
+            m = decimal.Decimal(str(m))
         self.opts['bounds'][0] = m
         if update:
             self.setValue()
@@ -300,7 +295,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
         Select the numerical portion of the text to allow quick editing by the user.
         """
         le = self.lineEdit()
-        text = asUnicode(le.text())
+        text = le.text()
         m = self.opts['regex'].match(text)
         if m is None:
             return
@@ -359,8 +354,8 @@ class SpinBox(QtGui.QAbstractSpinBox):
         if self.opts['int']:
             value = int(value)
 
-        if not isinstance(value, D):
-            value = D(asUnicode(value))
+        if not isinstance(value, decimal.Decimal):
+            value = decimal.Decimal(str(value))
 
         prev, self.val = self.val, value
         changed = not fn.eq(value, prev)  # use fn.eq to handle nan
@@ -400,8 +395,8 @@ class SpinBox(QtGui.QAbstractSpinBox):
         if isinf(self.val) or isnan(self.val):
             return
 
-        n = D(int(n))   ## n must be integral number of steps.
-        s = [D(-1), D(1)][n >= 0]  ## determine sign of step
+        n = decimal.Decimal(int(n))   ## n must be integral number of steps.
+        s = [decimal.Decimal(-1), decimal.Decimal(1)][n >= 0]  ## determine sign of step
         val = self.val
         
         for i in range(int(abs(n))):
@@ -410,11 +405,11 @@ class SpinBox(QtGui.QAbstractSpinBox):
                     step = self.opts['minStep']
                     exp = None
                 else:
-                    vs = [D(-1), D(1)][val >= 0]
-                    #exp = D(int(abs(val*(D('1.01')**(s*vs))).log10()))
-                    fudge = D('1.01')**(s*vs) ## fudge factor. at some places, the step size depends on the step sign.
+                    vs = [decimal.Decimal(-1), decimal.Decimal(1)][val >= 0]
+                    #exp = decimal.Decimal(int(abs(val*(decimal.Decimal('1.01')**(s*vs))).log10()))
+                    fudge = decimal.Decimal('1.01')**(s*vs) ## fudge factor. at some places, the step size depends on the step sign.
                     exp = abs(val * fudge).log10().quantize(1, decimal.ROUND_FLOOR)
-                    step = self.opts['step'] * D(10)**exp
+                    step = self.opts['step'] * decimal.Decimal(10)**exp
                 if 'minStep' in self.opts:
                     step = max(step, self.opts['minStep'])
                 val += s * step
@@ -423,7 +418,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
                 val += s*self.opts['step']
                 
             if 'minStep' in self.opts and abs(val) < self.opts['minStep']:
-                val = D(0)
+                val = decimal.Decimal(0)
         self.setValue(val, delaySignal=True)  ## note all steps (arrow buttons, wheel, up/down keys..) emit delayed signals only.
 
     def valueInRange(self, value):
@@ -560,9 +555,7 @@ class SpinBox(QtGui.QAbstractSpinBox):
 
     def editingFinishedEvent(self):
         """Edit has finished; set value."""
-        #print "Edit finished."
-        if asUnicode(self.lineEdit().text()) == self.lastText:
-            #print "no text change."
+        if self.lineEdit().text() == self.lastText:
             return
         try:
             val = self.interpret()
@@ -570,10 +563,8 @@ class SpinBox(QtGui.QAbstractSpinBox):
             return
         
         if val is False:
-            #print "value invalid:", str(self.lineEdit().text())
             return
         if val == self.val:
-            #print "no value change:", val, self.val
             return
         self.setValue(val, delaySignal=False)  ## allow text update so that values are reformatted pretty-like
 
@@ -593,12 +584,12 @@ class SpinBox(QtGui.QAbstractSpinBox):
         super().paintEvent(ev)
 
 
-class ErrorBox(QtGui.QWidget):
+class ErrorBox(QtWidgets.QWidget):
     """Red outline to draw around lineedit when value is invalid.
     (for some reason, setting border from stylesheet does not work)
     """
     def __init__(self, parent):
-        QtGui.QWidget.__init__(self, parent)
+        QtWidgets.QWidget.__init__(self, parent)
         parent.installEventFilter(self)
         self.setAttribute(QtCore.Qt.WidgetAttribute.WA_TransparentForMouseEvents)
         self._resize()
