@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
 import numpy as np
-from ..Qt import QtGui, QtCore
-from ..python2_3 import asUnicode, basestring
-from .. import metaarray
 
+from .. import metaarray
+from ..Qt import QtCore, QtGui, QtWidgets
+
+translate = QtCore.QCoreApplication.translate
 
 __all__ = ['TableWidget']
 
@@ -26,7 +26,7 @@ def _defersort(fn):
     return defersort
 
 
-class TableWidget(QtGui.QTableWidget):
+class TableWidget(QtWidgets.QTableWidget):
     """Extends QTableWidget with some useful functions for automatic data handling
     and copy / export context menu. Can automatically format and display a variety
     of data types (see :func:`setData() <pyqtgraph.TableWidget.setData>` for more
@@ -49,13 +49,13 @@ class TableWidget(QtGui.QTableWidget):
         ===================== =================================================
         """
         
-        QtGui.QTableWidget.__init__(self, *args)
+        QtWidgets.QTableWidget.__init__(self, *args)
         
         self.itemClass = TableWidgetItem
         
-        self.setVerticalScrollMode(self.ScrollPerPixel)
-        self.setSelectionMode(QtGui.QAbstractItemView.ContiguousSelection)
-        self.setSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
+        self.setVerticalScrollMode(self.ScrollMode.ScrollPerPixel)
+        self.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ContiguousSelection)
+        self.setSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
         self.clear()
         
         kwds.setdefault('sortable', True)
@@ -73,15 +73,15 @@ class TableWidget(QtGui.QTableWidget):
         
         self.itemChanged.connect(self.handleItemChanged)
         
-        self.contextMenu = QtGui.QMenu()
-        self.contextMenu.addAction('Copy Selection').triggered.connect(self.copySel)
-        self.contextMenu.addAction('Copy All').triggered.connect(self.copyAll)
-        self.contextMenu.addAction('Save Selection').triggered.connect(self.saveSel)
-        self.contextMenu.addAction('Save All').triggered.connect(self.saveAll)
+        self.contextMenu = QtWidgets.QMenu()
+        self.contextMenu.addAction(translate("TableWidget", 'Copy Selection')).triggered.connect(self.copySel)
+        self.contextMenu.addAction(translate("TableWidget", 'Copy All')).triggered.connect(self.copyAll)
+        self.contextMenu.addAction(translate("TableWidget", 'Save Selection')).triggered.connect(self.saveSel)
+        self.contextMenu.addAction(translate("TableWidget", 'Save All')).triggered.connect(self.saveAll)
         
     def clear(self):
         """Clear all contents from the table."""
-        QtGui.QTableWidget.clear(self)
+        QtWidgets.QTableWidget.clear(self)
         self.verticalHeadersSet = False
         self.horizontalHeadersSet = False
         self.items = []
@@ -93,12 +93,12 @@ class TableWidget(QtGui.QTableWidget):
         """Set the data displayed in the table.
         Allowed formats are:
         
-        * numpy arrays
-        * numpy record arrays 
-        * metaarrays
-        * list-of-lists  [[1,2,3], [4,5,6]]
-        * dict-of-lists  {'x': [1,2,3], 'y': [4,5,6]}
-        * list-of-dicts  [{'x': 1, 'y': 4}, {'x': 2, 'y': 5}, ...]
+          * numpy arrays
+          * numpy record arrays
+          * metaarrays
+          * list-of-lists  [[1,2,3], [4,5,6]]
+          * dict-of-lists  {'x': [1,2,3], 'y': [4,5,6]}
+          * list-of-dicts  [{'x': 1, 'y': 4}, {'x': 2, 'y': 5}, ...]
         """
         self.clear()
         self.appendData(data)
@@ -148,7 +148,7 @@ class TableWidget(QtGui.QTableWidget):
             
         if (self._sorting and self.horizontalHeadersSet and 
             self.horizontalHeader().sortIndicatorSection() >= self.columnCount()):
-            self.sortByColumn(0, QtCore.Qt.AscendingOrder)
+            self.sortByColumn(0, QtCore.Qt.SortOrder.AscendingOrder)
     
     def setEditable(self, editable=True):
         self.editable = editable
@@ -169,7 +169,7 @@ class TableWidget(QtGui.QTableWidget):
         Added in version 0.9.9.
         
         """
-        if format is not None and not isinstance(format, basestring) and not callable(format):
+        if format is not None and not isinstance(format, str) and not callable(format):
             raise ValueError("Format argument must string, callable, or None. (got %s)" % format)
         
         self._formats[column] = format
@@ -202,19 +202,19 @@ class TableWidget(QtGui.QTableWidget):
         if isinstance(data, list) or isinstance(data, tuple):
             return lambda d: d.__iter__(), None
         elif isinstance(data, dict):
-            return lambda d: iter(d.values()), list(map(asUnicode, data.keys()))
+            return lambda d: iter(d.values()), list(map(str, data.keys()))
         elif (hasattr(data, 'implements') and data.implements('MetaArray')):
             if data.axisHasColumns(0):
-                header = [asUnicode(data.columnName(0, i)) for i in range(data.shape[0])]
+                header = [str(data.columnName(0, i)) for i in range(data.shape[0])]
             elif data.axisHasValues(0):
-                header = list(map(asUnicode, data.xvals(0)))
+                header = list(map(str, data.xvals(0)))
             else:
                 header = None
             return self.iterFirstAxis, header
         elif isinstance(data, np.ndarray):
             return self.iterFirstAxis, None
         elif isinstance(data, np.void):
-            return self.iterate, list(map(asUnicode, data.dtype.names))
+            return self.iterate, list(map(str, data.dtype.names))
         elif data is None:
             return (None,None)
         elif np.isscalar(data):
@@ -310,22 +310,22 @@ class TableWidget(QtGui.QTableWidget):
         if self.horizontalHeadersSet:
             row = []
             if self.verticalHeadersSet:
-                row.append(asUnicode(''))
+                row.append('')
             
             for c in columns:
-                row.append(asUnicode(self.horizontalHeaderItem(c).text()))
+                row.append(self.horizontalHeaderItem(c).text())
             data.append(row)
         
         for r in rows:
             row = []
             if self.verticalHeadersSet:
-                row.append(asUnicode(self.verticalHeaderItem(r).text()))
+                row.append(self.verticalHeaderItem(r).text())
             for c in columns:
                 item = self.item(r, c)
                 if item is not None:
-                    row.append(asUnicode(item.value))
+                    row.append(str(item.value))
                 else:
-                    row.append(asUnicode(''))
+                    row.append('')
             data.append(row)
             
         s = ''
@@ -335,11 +335,11 @@ class TableWidget(QtGui.QTableWidget):
 
     def copySel(self):
         """Copy selected data to clipboard."""
-        QtGui.QApplication.clipboard().setText(self.serialize(useSelection=True))
+        QtWidgets.QApplication.clipboard().setText(self.serialize(useSelection=True))
 
     def copyAll(self):
         """Copy all data to clipboard."""
-        QtGui.QApplication.clipboard().setText(self.serialize(useSelection=False))
+        QtWidgets.QApplication.clipboard().setText(self.serialize(useSelection=False))
 
     def saveSel(self):
         """Save selected data to file."""
@@ -350,7 +350,12 @@ class TableWidget(QtGui.QTableWidget):
         self.save(self.serialize(useSelection=False))
 
     def save(self, data):
-        fileName = QtGui.QFileDialog.getSaveFileName(self, "Save As..", "", "Tab-separated values (*.tsv)")
+        fileName = QtWidgets.QFileDialog.getSaveFileName(
+            self,
+            f"{translate('TableWidget', 'Save As')}...",
+            "",
+            f"{translate('TableWidget', 'Tab-separated values')} (*.tsv)"
+        )
         if isinstance(fileName, tuple):
             fileName = fileName[0]  # Qt4/5 API difference
         if fileName == '':
@@ -362,25 +367,25 @@ class TableWidget(QtGui.QTableWidget):
         self.contextMenu.popup(ev.globalPos())
         
     def keyPressEvent(self, ev):
-        if ev.key() == QtCore.Qt.Key_C and ev.modifiers() == QtCore.Qt.ControlModifier:
+        if ev.matches(QtGui.QKeySequence.StandardKey.Copy):
             ev.accept()
             self.copySel()
         else:
-            QtGui.QTableWidget.keyPressEvent(self, ev)
+            super().keyPressEvent(ev)
 
     def handleItemChanged(self, item):
         item.itemChanged()
 
 
-class TableWidgetItem(QtGui.QTableWidgetItem):
+class TableWidgetItem(QtWidgets.QTableWidgetItem):
     def __init__(self, val, index, format=None):
-        QtGui.QTableWidgetItem.__init__(self, '')
+        QtWidgets.QTableWidgetItem.__init__(self, '')
         self._blockValueChange = False
         self._format = None
         self._defaultFormat = '%0.3g'
         self.sortMode = 'value'
         self.index = index
-        flags = QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled
+        flags = QtCore.Qt.ItemFlag.ItemIsSelectable | QtCore.Qt.ItemFlag.ItemIsEnabled
         self.setFlags(flags)
         self.setValue(val)
         self.setFormat(format)
@@ -390,9 +395,9 @@ class TableWidgetItem(QtGui.QTableWidgetItem):
         Set whether this item is user-editable.
         """
         if editable:
-            self.setFlags(self.flags() | QtCore.Qt.ItemIsEditable)
+            self.setFlags(self.flags() | QtCore.Qt.ItemFlag.ItemIsEditable)
         else:
-            self.setFlags(self.flags() & ~QtCore.Qt.ItemIsEditable)
+            self.setFlags(self.flags() & ~QtCore.Qt.ItemFlag.ItemIsEditable)
             
     def setSortMode(self, mode):
         """
@@ -421,7 +426,7 @@ class TableWidgetItem(QtGui.QTableWidgetItem):
         
         Added in version 0.9.9.
         """
-        if fmt is not None and not isinstance(fmt, basestring) and not callable(fmt):
+        if fmt is not None and not isinstance(fmt, str) and not callable(fmt):
             raise ValueError("Format argument must string, callable, or None. (got %s)" % fmt)
         self._format = fmt
         self._updateText()
@@ -467,7 +472,7 @@ class TableWidgetItem(QtGui.QTableWidgetItem):
             else:
                 return self._format % self.value
         else:
-            return asUnicode(self.value)
+            return str(self.value)
 
     def __lt__(self, other):
         if self.sortMode == 'index' and hasattr(other, 'index'):
@@ -479,8 +484,8 @@ class TableWidgetItem(QtGui.QTableWidgetItem):
 
 
 if __name__ == '__main__':
-    app = QtGui.QApplication([])
-    win = QtGui.QMainWindow()
+    app = QtWidgets.QApplication([])
+    win = QtWidgets.QMainWindow()
     t = TableWidget()
     win.setCentralWidget(t)
     win.resize(800,600)
