@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
-from ..Qt import QtGui, QtCore
 import warnings
 
+from ..Qt import QtCore, QtGui, QtWidgets
+
 __all__ = ['VerticalLabel']
-#class VerticalLabel(QtGui.QLabel):
+#class VerticalLabel(QtWidgets.QLabel):
     #def paintEvent(self, ev):
         #p = QtGui.QPainter(self)
         #p.rotate(-90)
@@ -18,9 +18,9 @@ __all__ = ['VerticalLabel']
         #else:
             #return QtCore.QSize(16, 50)
 
-class VerticalLabel(QtGui.QLabel):
+class VerticalLabel(QtWidgets.QLabel):
     def __init__(self, text, orientation='vertical', forceWidth=True):
-        QtGui.QLabel.__init__(self, text)
+        QtWidgets.QLabel.__init__(self, text)
         self.forceWidth = forceWidth
         self.orientation = None
         self.setOrientation(orientation)
@@ -83,10 +83,10 @@ class VerticalLabel(QtGui.QLabel):
 
 
 if __name__ == '__main__':
-    app = QtGui.QApplication([])
-    win = QtGui.QMainWindow()
-    w = QtGui.QWidget()
-    l = QtGui.QGridLayout()
+    app = QtWidgets.QApplication([])
+    win = QtWidgets.QMainWindow()
+    w = QtWidgets.QWidget()
+    l = QtWidgets.QGridLayout()
     w.setLayout(l)
     
     l1 = VerticalLabel("text 1", orientation='horizontal')
@@ -98,4 +98,4 @@ if __name__ == '__main__':
     l.addWidget(l3, 2, 2)
     l.addWidget(l4, 3, 3)
     win.setCentralWidget(w)
-    win.show()
\ No newline at end of file
+    win.show()
